LABEL_ROW_METADATA_BLURB = [
    {
        "label_hash": "44e1bdfb-3c45-4edb-8a71-a969f22fd632",
        "task_uuid": "ec176f56-6156-400d-97a2-68e8707e89a1",
        "branch_name": "main",
        "label_status": "LABEL_IN_PROGRESS",
        "created_at": "2023-03-21T09:58:08.472959",
        "last_edited_at": "2023-03-21T11:03:52.573279",
        "data_hash": "9b45984a-0046-4389-ba0d-b73b6b35ee82",
        "dataset_hash": "6a8a9b04-b173-44a7-b746-558eaa468132",
        "dataset_title": "Test dataset",
        "data_title": "sample.mp4",
        "data_type": "VIDEO",
        "is_shadow_data": False,
        "frames_per_second": 23.976023976023978,
        "duration": 29.07071,
        "number_of_frames": 697,
        "data_link": "cord-videos-dev/CcJU09GkdjcLCslKtd6bEqi48ZA2/9b45984a-0046-4389-ba0d-b73b6b35ee82",
        "file_type": "video/mp4",
        "file_size": "5468354.0",
        "width": 1280,
        "height": 720,
        "is_optimised_image_group": "None",
        "is_image_sequence": "None",
        "annotation_task_status": "ASSIGNED",
    },
    {
        "label_hash": "2321fd7d-99cd-4d85-b371-9a9f59cf79a7",
        "task_uuid": "ec176f56-6156-400d-97a2-68e8707e89a2",
        "branch_name": "main",
        "label_status": "LABEL_IN_PROGRESS",
        "created_at": "2023-03-21T10:25:48.821571",
        "last_edited_at": "2023-03-21T11:51:19.785354",
        "data_hash": "b13c4f55-1eae-4832-93e8-83b030ef17b0",
        "dataset_hash": "6a8a9b04-b173-44a7-b746-558eaa468132",
        "dataset_title": "Test dataset",
        "data_title": "Screenshot 2023-03-16 at 13.04.59.png",
        "data_type": "IMAGE",
        "is_shadow_data": False,
        "frames_per_second": "None",
        "duration": "None",
        "number_of_frames": 1,
        "data_link": "cord-images-dev/CcJU09GkdjcLCslKtd6bEqi48ZA2/b13c4f55-1eae-4832-93e8-83b030ef17b0",
        "file_type": "image/png",
        "file_size": "13154.0",
        "width": 304,
        "height": 220,
        "is_optimised_image_group": "None",
        "is_image_sequence": "None",
        "annotation_task_status": "QUEUED",
    },
    {
        "label_hash": "f6dcda1a-8e0a-45b2-a3c5-d7ebfe93ad8a",
        "task_uuid": "ec176f56-6156-400d-97a2-68e8707e89a3",
        "branch_name": "main",
        "label_status": "LABEL_IN_PROGRESS",
        "created_at": "2023-03-21T12:02:42.287150",
        "last_edited_at": "2023-03-21T12:02:42.287150",
        "data_hash": "f53546bf-9c1e-47d5-bc20-282a4c187977",
        "dataset_hash": "6a8a9b04-b173-44a7-b746-558eaa468132",
        "dataset_title": "Test dataset",
        "data_title": "mega-image-group",
        "data_type": "IMG_GROUP",
        "is_shadow_data": False,
        "frames_per_second": "None",
        "duration": "None",
        "number_of_frames": 4,
        "data_link": "None",
        "file_type": "None",
        "file_size": "None",
        "width": "None",
        "height": "None",
        "is_optimised_image_group": False,
        "is_image_sequence": False,
        "annotation_task_status": "QUEUED",
    },
]

LABEL_ROW_BLURB = [
    {
        "label_hash": "2321fd7d-99cd-4d85-b371-9a9f59cf79a7",
        "branch_name": "main",
        "created_at": "2023-03-21 10:25:48",
        "last_edited_at": "2023-03-21 11:51:19",
        "dataset_hash": "6a8a9b04-b173-44a7-b746-558eaa468132",
        "dataset_title": "Test dataset",
        "data_title": "Screenshot 2023-03-16 at 13.04.59.png",
        "data_hash": "b13c4f55-1eae-4832-93e8-83b030ef17b0",
        "data_type": "image",
        "is_image_sequence": "None",
        "video_link": "None",
        "data_units": {
            "b13c4f55-1eae-4832-93e8-83b030ef17b0": {
                "data_hash": "b13c4f55-1eae-4832-93e8-83b030ef17b0",
                "data_title": "Screenshot 2023-03-16 at 13.04.59.png",
                "data_type": "image/png",
                "data_sequence": 0,
                "labels": {
                    "objects": [
                        {
                            "name": "Pear",
                            "color": "#E27300",
                            "shape": "bounding_box",
                            "value": "pear",
                            "createdAt": "Tue, 21 Mar 2023 11:21:51 GMT",
                            "createdBy": "sergei@encord.com",
                            "confidence": 1,
                            "objectHash": "8/VCHNzS",
                            "featureHash": "no8RyWRY",
                            "manualAnnotation": True,
                            "boundingBox": {"h": 0.1836, "w": 0.093, "x": 0.2051, "y": 0.3293},
                        },
                        {
                            "name": "Pear",
                            "color": "#E27300",
                            "shape": "bounding_box",
                            "value": "pear",
                            "createdAt": "Tue, 21 Mar 2023 11:21:56 GMT",
                            "createdBy": "sergei@encord.com",
                            "confidence": 1,
                            "objectHash": "uGsbuPaf",
                            "featureHash": "no8RyWRY",
                            "manualAnnotation": True,
                            "boundingBox": {"h": 0.178, "w": 0.0903, "x": 0.4376, "y": 0.3367},
                        },
                    ],
                    "classifications": [
                        {
                            "name": "hot dog",
                            "value": "hot_dog",
                            "createdAt": "Tue, 21 Mar 2023 11:23:36 GMT",
                            "createdBy": "sergei@encord.com",
                            "confidence": 1,
                            "featureHash": "4BjoumKd",
                            "lastEditedAt": "Tue, 21 Mar 2023 11:23:36 GMT",
                            "classificationHash": "j8oMRZ8f",
                            "manualAnnotation": True,
                        }
                    ],
                },
                "data_link": "https://storage.googleapis.com/encord-local-dev.appspot.com/cord-images-dev/CcJU09GkdjcLCslKtd6bEqi48ZA2/b13c4f55-1eae-4832-93e8-83b030ef17b0?X-Goog-Algorithm=GOOG4-RSA-SHA256&X-Goog-Credential=firebase-adminsdk-efw44%40encord-local-dev.iam.gserviceaccount.com%2F20230328%2Fauto%2Fstorage%2Fgoog4_request&X-Goog-Date=20230328T153206Z&X-Goog-Expires=604800&X-Goog-SignedHeaders=host&X-Goog-Signature=69a8f2fca41edc7d81d3d2719e3ad188cee29fb5ac3302f24e0e93cf60cb6ec38df6b39203ede30bca981c347722345ed13848a22fe0b63da9de44cd5ecb5b4d85a11ac595d927df7b354a93a5523bb1282cf8b4fa5abb3d2c5e41112f0002faf5c07d577f9241d6d23695c503c7a34b12733bea4827d1e6c5a4c4797607e47193beebb038e2debfd650033df9ed90db44e2aa853f10a1cfd55d7e288685e9c6b0e4812993fd089f114e7d8f7bfb78e4fae52b8946d6946807aba148b5514f9855d669f1e476cd3315ed699377c7db5947358b96666fd0b6c24eb47bb1f354fc70b48644a91427922927b9db85353d04f4024e71fa6cc162a6a591add123b814",
                "width": 304,
                "height": 220,
            }
        },
        "object_answers": {
            "8/VCHNzS": {"objectHash": "8/VCHNzS", "classifications": []},
            "uGsbuPaf": {"objectHash": "uGsbuPaf", "classifications": []},
        },
        "classification_answers": {
            "j8oMRZ8f": {
                "classificationHash": "j8oMRZ8f",
                "classifications": [
                    {
                        "name": "hot dog",
                        "value": "hot_dog",
                        "answers": [{"name": "not hot dog", "value": "not_hot_dog", "featureHash": "MpKwvogL"}],
                        "featureHash": "t/prlg4Z",
                        "manualAnnotation": True,
                    }
                ],
            }
        },
        "object_actions": {},
        "label_status": "LABEL_IN_PROGRESS",
        "export_history": [
            {
                "export_hash": "a5570813-ad77-4f4d-b507-bdda3ff4db8d",
                "exported_at": "Tue, 21 Mar 2023 10:25:49 UTC",
                "user_email": "sergei@encord.com",
            }
        ],
        "export_hash": "56042be4-3c8e-4a6e-a916-37d31da58617",
        "exported_at": "Tue, 28 Mar 2023 14:32:14 UTC",
        "annotation_task_status": "QUEUED",
    },
    {
        "label_hash": "f6dcda1a-8e0a-45b2-a3c5-d7ebfe93ad8a",
        "branch_name": "main",
        "created_at": "2023-03-21 12:02:42",
        "last_edited_at": "2023-03-21 12:02:42",
        "dataset_hash": "6a8a9b04-b173-44a7-b746-558eaa468132",
        "dataset_title": "Test dataset",
        "data_title": "mega-image-group",
        "data_hash": "f53546bf-9c1e-47d5-bc20-282a4c187977",
        "data_type": "img_group",
        "is_image_sequence": False,
        "video_link": "None",
        "data_units": {
            "b06893e3-d7d2-4aa9-83b0-d91c9acc51be": {
                "data_hash": "b06893e3-d7d2-4aa9-83b0-d91c9acc51be",
                "data_title": "Screenshot 2023-03-14 at 11.41.48.png",
                "data_link": "https://storage.googleapis.com/encord-local-dev.appspot.com/cord-images-dev/CcJU09GkdjcLCslKtd6bEqi48ZA2/3fb973bf-50fc-4428-8149-c0b7cd23d92f?X-Goog-Algorithm=GOOG4-RSA-SHA256&X-Goog-Credential=firebase-adminsdk-efw44%40encord-local-dev.iam.gserviceaccount.com%2F20230328%2Fauto%2Fstorage%2Fgoog4_request&X-Goog-Date=20230328T153207Z&X-Goog-Expires=604800&X-Goog-SignedHeaders=host&X-Goog-Signature=9a079800c779ab1242a7f7e6115155769199a40af88df53339ad38212171e8d7ea70fd5c82376dd2c90767f6e06f72b6cd728ee1d455a9e4b9180f213e2b6e94e3526d60f1877cf419adae20435ec8a2a44629c3dc2166fc0871c9cff9cd9f8cdec5ffe2ba1548ea5d9b276697a9c347b7a728de32cb6bec89708cd9b38d4330bbc81f28f128d3a0f18ac7036137b77be385cce4f422916e08b21fd4db8670d26e149ffff266356b2884c39f278f2ee196bfaff5dd91db8bb1f0a049a30a917aef3ca8e73219c3dbea572e0ea3c518bada6020d2bbb265b9bf64897070b1abbb323c10a752e99d60a828b4169a1da3629db4f4c2b66c20424689fccdd4fcf8b6",
                "data_type": "image/png",
                "data_sequence": "0",
                "width": 376,
                "height": 304,
                "labels": {"objects": [], "classifications": []},
            },
            "714a7e19-5802-447a-bfa8-8850ff906999": {
                "data_hash": "714a7e19-5802-447a-bfa8-8850ff906999",
                "data_title": "Screenshot 2023-03-14 at 11.41.49.png",
                "data_link": "https://storage.googleapis.com/encord-local-dev.appspot.com/cord-images-dev/CcJU09GkdjcLCslKtd6bEqi48ZA2/47121fd5-fce3-4688-9027-81bafb9060e4?X-Goog-Algorithm=GOOG4-RSA-SHA256&X-Goog-Credential=firebase-adminsdk-efw44%40encord-local-dev.iam.gserviceaccount.com%2F20230328%2Fauto%2Fstorage%2Fgoog4_request&X-Goog-Date=20230328T153207Z&X-Goog-Expires=604800&X-Goog-SignedHeaders=host&X-Goog-Signature=2ef78f7ff647325221c6d01ca41417e6ae072ec0b1370da3fedae85c594dd0ed3fb0da2f7a11973e0f71f8702cf33ac06fbecf1ddaa4dd594cc46e5e0876563e978f4ae7450bf6992aa568715038e8f2d590ed521e1732d727250cfe9df81c9ec0534c05a61b85562f05a5d53d6e33f5e309cd2fbf02f2b0638d0030d0c553cbae0c480b11da23082d740703202813e00f7301297a37070b6fce8f63c9f6ad1845e70a3b36fae48b63a9ce572fc14bc5907a3efd881076ff9fbc7eee16e9d9e51427d5c289b49edf82279419d96e561288e59653a55f816f9a3723bb767acc6c0b8d06c059303258ac19d7468c72da9e4729320304a89a774641cc1c5d07fe03",
                "data_type": "image/png",
                "data_sequence": "1",
                "width": 418,
                "height": 258,
                "labels": {"objects": [], "classifications": []},
            },
            "7b1dcb67-ca49-421b-9fc5-1fa52f538aab": {
                "data_hash": "7b1dcb67-ca49-421b-9fc5-1fa52f538aab",
                "data_title": "Screenshot 2023-03-16 at 13.04.51.png",
                "data_link": "https://storage.googleapis.com/encord-local-dev.appspot.com/cord-images-dev/CcJU09GkdjcLCslKtd6bEqi48ZA2/8f998937-8a24-413a-b332-c34fd7e45de1?X-Goog-Algorithm=GOOG4-RSA-SHA256&X-Goog-Credential=firebase-adminsdk-efw44%40encord-local-dev.iam.gserviceaccount.com%2F20230328%2Fauto%2Fstorage%2Fgoog4_request&X-Goog-Date=20230328T153207Z&X-Goog-Expires=604800&X-Goog-SignedHeaders=host&X-Goog-Signature=3cc9e75481df91885530ae08fc5ecb2a13fb9829abca778464ad33eb2c1a87280cfd6997450c5b35786184882aee7b81688d45d49a6f0fa7c15c8664145e47d80b0b267ab5ea00538fe02711372fa30bc25887fe2b8a5265a15c0e2b0a42e5d61238ade7291386324d8bc28d0fd520016170491ec09bc98bd69ac27c1d99e55a00c4bde63c7bf89087aebea58a7f239432ef5d648bc2b48a55d052a784fb74fcbd98b322740e6c53b23a59577c3e77e462d51a8e16c5837f6ec9a8b542d8178c7a96a2a8b2e1734facde0f7d003e2397d0b7fead1716602443bfa5c7f356ab5be9744e53e0534353b89bc0ee46e2cad65efa4e63ac2ad2f90920626f8a4383ef",
                "data_type": "image/png",
                "data_sequence": "2",
                "width": 226,
                "height": 144,
                "labels": {"objects": [], "classifications": []},
            },
            "041408a2-c887-451b-a961-872f6a319e41": {
                "data_hash": "041408a2-c887-451b-a961-872f6a319e41",
                "data_title": "Screenshot 2023-03-16 at 13.04.58.png",
                "data_link": "https://storage.googleapis.com/encord-local-dev.appspot.com/cord-images-dev/CcJU09GkdjcLCslKtd6bEqi48ZA2/a856107e-af19-46d3-8453-6b0d19a93b0e?X-Goog-Algorithm=GOOG4-RSA-SHA256&X-Goog-Credential=firebase-adminsdk-efw44%40encord-local-dev.iam.gserviceaccount.com%2F20230328%2Fauto%2Fstorage%2Fgoog4_request&X-Goog-Date=20230328T153207Z&X-Goog-Expires=604800&X-Goog-SignedHeaders=host&X-Goog-Signature=2f3af79a9467c9a0978654eda9588d65c01ed49205b47cad76d48475ac622974e28632050b597b7902367b3ab8d970688e2c46dfe009c03825ec82c4d968d2cbe8ada3c919cda4afe6991d943defa567e657fed9ff79cadce6ab55c7b5fbed5ba481dbd0f7917378ceeebecb8c6f194c4508c21bd3c87a5e7c92ad1c23af5484ea953f65ce979280414b33589f4a9128a26664c8dcdd6546e0603c4e66087b622d35d41187326fba585117350817e49459df0ab0a4a600d7bb4d7da4cf16ff21f41532a382014903ae9eb7737d7e0e54c78293ba8e92e3a9095ee9e296d3f1e73558464c8daa92f484423c88051ace24d7551cb77f7d5df078355a81818d6186",
                "data_type": "image/png",
                "data_sequence": "3",
                "width": 252,
                "height": 294,
                "labels": {"objects": [], "classifications": []},
            },
        },
        "object_answers": {},
        "classification_answers": {},
        "object_actions": {},
        "label_status": "LABEL_IN_PROGRESS",
        "export_history": [],
        "export_hash": "9778ee89-6c31-40ec-a67c-61bba6fcbb49",
        "exported_at": "Tue, 28 Mar 2023 14:32:15 UTC",
        "annotation_task_status": "QUEUED",
    },
    {
        "label_hash": "44e1bdfb-3c45-4edb-8a71-a969f22fd632",
        "branch_name": "main",
        "created_at": "2023-03-21 09:58:08",
        "last_edited_at": "2023-03-22 19:02:27",
        "dataset_hash": "6a8a9b04-b173-44a7-b746-558eaa468132",
        "dataset_title": "Test dataset",
        "data_title": "sample.mp4",
        "data_hash": "9b45984a-0046-4389-ba0d-b73b6b35ee82",
        "data_type": "video",
        "is_image_sequence": "None",
        "video_link": "https://storage.googleapis.com/encord-local-dev.appspot.com/cord-videos-dev/CcJU09GkdjcLCslKtd6bEqi48ZA2/9b45984a-0046-4389-ba0d-b73b6b35ee82?X-Goog-Algorithm=GOOG4-RSA-SHA256&X-Goog-Credential=firebase-adminsdk-efw44%40encord-local-dev.iam.gserviceaccount.com%2F20230328%2Fauto%2Fstorage%2Fgoog4_request&X-Goog-Date=20230328T153208Z&X-Goog-Expires=604800&X-Goog-SignedHeaders=host&X-Goog-Signature=07aa2e0a51f13576be1be021072fd283f080b1bd26fb839a56b06165338bf3733b0051cce21eb27656728dcf8785f682d432781f3d1d638aa3bb150a492a62a20990449eaf2e02059d0ab835f67e9449c8637e47f36fbc1c0610eb06905a5b4cd1788df76228cf98f8beaf2e8b960374b5d40c90a062ecd9d49420829a6c4c78ee2ede62af00d2addc988488ffd638f633493fb4a14bf2067a6a036f90af728aef32700b5254d52026c69b30524cbe70959fae66ee5b6265579cfc6074ded8db9bfa8ca5ed432c2ad8ae9c1996a3b29efbcabf5cde024cce0a3ed25a69ff52b3c593bade583ef305a2f023585732e8c4362939d4e5a572faeb0492a41d574a2d",
        "data_units": {
            "9b45984a-0046-4389-ba0d-b73b6b35ee82": {
                "data_hash": "9b45984a-0046-4389-ba0d-b73b6b35ee82",
                "data_title": "sample.mp4",
                "data_type": "video/mp4",
                "data_sequence": 0,
                "labels": {
                    "0": {"objects": [], "classifications": []},
                    "1": {
                        "objects": [
                            {
                                "name": "Apple",
                                "color": "#D33115",
                                "shape": "bounding_box",
                                "value": "apple",
                                "createdAt": "Wed, 22 Mar 2023 18:00:13 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "objectHash": "cY1cfw0G",
                                "featureHash": "qWm5E3j4",
                                "manualAnnotation": True,
                                "boundingBox": {"h": 0.2712, "w": 0.1549, "x": 0.0238, "y": 0.3007},
                            },
                            {
                                "name": "Apple",
                                "color": "#D33115",
                                "shape": "bounding_box",
                                "value": "apple",
                                "createdAt": "Wed, 22 Mar 2023 18:00:17 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "objectHash": "IFgHza0m",
                                "featureHash": "qWm5E3j4",
                                "manualAnnotation": True,
                                "boundingBox": {"h": 0.2589, "w": 0.1664, "x": 0.4353, "y": 0.4302},
                            },
                            {
                                "name": "Apple",
                                "color": "#D33115",
                                "shape": "bounding_box",
                                "value": "apple",
                                "createdAt": "Wed, 22 Mar 2023 18:00:21 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "objectHash": "obAZPhln",
                                "featureHash": "qWm5E3j4",
                                "manualAnnotation": True,
                                "boundingBox": {"h": 0.3185, "w": 0.1884, "x": 0.6087, "y": 0.3993},
                            },
                        ],
                        "classifications": [],
                    },
                    "2": {"objects": [], "classifications": []},
                    "290": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "291": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "292": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "293": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "294": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "295": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "296": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "297": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "298": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "299": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "300": {
                        "objects": [
                            {
                                "name": "Apple",
                                "color": "#D33115",
                                "shape": "bounding_box",
                                "value": "apple",
                                "createdAt": "Tue, 21 Mar 2023 10:29:55 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "objectHash": "cY1cfw0G",
                                "featureHash": "qWm5E3j4",
                                "manualAnnotation": True,
                                "boundingBox": {"h": 0.0944, "w": 0.0526, "x": 0.2919, "y": 0.8575},
                            },
                            {
                                "name": "Apple",
                                "color": "#D33115",
                                "shape": "bounding_box",
                                "value": "apple",
                                "createdAt": "Tue, 21 Mar 2023 10:30:04 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "objectHash": "IFgHza0m",
                                "featureHash": "qWm5E3j4",
                                "manualAnnotation": True,
                                "boundingBox": {"h": 0.0834, "w": 0.0541, "x": 0.4507, "y": 0.8722},
                            },
                            {
                                "name": "Apple",
                                "color": "#D33115",
                                "shape": "bounding_box",
                                "value": "apple",
                                "createdAt": "Tue, 21 Mar 2023 10:30:09 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "objectHash": "obAZPhln",
                                "featureHash": "qWm5E3j4",
                                "manualAnnotation": True,
                                "boundingBox": {"h": 0.0972, "w": 0.0541, "x": 0.5899, "y": 0.8722},
                            },
                        ],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "301": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "302": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "303": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "304": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "305": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "306": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "307": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "308": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "309": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                    "310": {
                        "objects": [],
                        "classifications": [
                            {
                                "name": "hot dog",
                                "value": "hot_dog",
                                "createdAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "createdBy": "sergei@encord.com",
                                "confidence": 1,
                                "featureHash": "4BjoumKd",
                                "lastEditedAt": "Tue, 21 Mar 2023 10:33:59 GMT",
                                "classificationHash": "+GqFZjI0",
                                "manualAnnotation": True,
                            }
                        ],
                    },
                },
                "data_fps": 23.976023976023978,
                "data_duration": 29.07071,
                "data_link": "https://storage.googleapis.com/encord-local-dev.appspot.com/cord-videos-dev/CcJU09GkdjcLCslKtd6bEqi48ZA2/9b45984a-0046-4389-ba0d-b73b6b35ee82?X-Goog-Algorithm=GOOG4-RSA-SHA256&X-Goog-Credential=firebase-adminsdk-efw44%40encord-local-dev.iam.gserviceaccount.com%2F20230328%2Fauto%2Fstorage%2Fgoog4_request&X-Goog-Date=20230328T153208Z&X-Goog-Expires=604800&X-Goog-SignedHeaders=host&X-Goog-Signature=07aa2e0a51f13576be1be021072fd283f080b1bd26fb839a56b06165338bf3733b0051cce21eb27656728dcf8785f682d432781f3d1d638aa3bb150a492a62a20990449eaf2e02059d0ab835f67e9449c8637e47f36fbc1c0610eb06905a5b4cd1788df76228cf98f8beaf2e8b960374b5d40c90a062ecd9d49420829a6c4c78ee2ede62af00d2addc988488ffd638f633493fb4a14bf2067a6a036f90af728aef32700b5254d52026c69b30524cbe70959fae66ee5b6265579cfc6074ded8db9bfa8ca5ed432c2ad8ae9c1996a3b29efbcabf5cde024cce0a3ed25a69ff52b3c593bade583ef305a2f023585732e8c4362939d4e5a572faeb0492a41d574a2d",
                "width": 1280,
                "height": 720,
            }
        },
        "object_answers": {
            "cY1cfw0G": {"objectHash": "cY1cfw0G", "classifications": []},
            "IFgHza0m": {"objectHash": "IFgHza0m", "classifications": []},
            "obAZPhln": {"objectHash": "obAZPhln", "classifications": []},
        },
        "classification_answers": {
            "+GqFZjI0": {
                "classificationHash": "+GqFZjI0",
                "classifications": [
                    {
                        "name": "hot dog",
                        "value": "hot_dog",
                        "answers": [{"name": "hot dog", "value": "hot_dog", "featureHash": "8fjrfCJy"}],
                        "featureHash": "t/prlg4Z",
                        "manualAnnotation": True,
                    }
                ],
            }
        },
        "object_actions": {},
        "label_status": "LABEL_IN_PROGRESS",
        "export_history": [
            {
                "export_hash": "b8354f9c-26f4-46ac-8207-1ff72b2552f4",
                "exported_at": "Tue, 21 Mar 2023 10:23:19 UTC",
                "user_email": "sergei@encord.com",
            }
        ],
        "export_hash": "069e80bc-3325-475f-8051-434aef7bc87b",
        "exported_at": "Tue, 28 Mar 2023 14:32:16 UTC",
        "annotation_task_status": "ASSIGNED",
    },
]

LABEL_ROW_METADATA_BLURB.sort(key=lambda x: x["data_hash"])
LABEL_ROW_BLURB.sort(key=lambda x: x["data_hash"])



"""
Code Block Name: Polygons to Bounding Boxes (No Typer)
"""

import logging
import time
from pathlib import Path
from typing import cast

import cv2
import numpy as np
from tqdm import tqdm, trange

from encord import EncordUserClient, Project
from encord.objects import LabelRowV2, Object, Shape
from encord.objects.coordinates import BoundingBoxCoordinates, PolygonCoordinates

MAX_NUM_EXECUTION_ATTEMPTS = 10


def try_execute(func, kwargs=None):
    for n in range(MAX_NUM_EXECUTION_ATTEMPTS):
        try:
            if kwargs:
                return func(**kwargs)
            else:
                return func()
        except Exception as e:
            logging.warning(
                f"Handling {e} when executing {func} with args {kwargs}.\n Trying again, attempt number {n + 1}."
            )
            time.sleep(0.5 * MAX_NUM_EXECUTION_ATTEMPTS)  # Linear backoff
    raise Exception("Reached maximum number of execution attempts.")


def initialize_label_rows(project: Project, batch_size: int = 200, include_unlabeled: bool = False) -> list[LabelRowV2]:
    label_rows = [lr for lr in project.list_label_rows_v2() if include_unlabeled or lr.label_hash is not None]
    for start in trange(
        0,
        len(label_rows),
        batch_size,
        desc=f"Initializing label rows [{project.title}]",
    ):
        bundle = project.create_bundle()
        for lr in label_rows[start : start + batch_size]:
            lr.initialise_labels(bundle=bundle)
        try_execute(bundle.execute)
    return label_rows


def populate_ontology_of_target_project(
    client: EncordUserClient, source: Project, target: Project
) -> dict[str, Object]:
    # Update the target ontology with potentially missing items
    source_ontology = source.ontology_structure
    target_ontology = client.get_ontology(target.ontology_hash)

    ontology_lookup: dict[str, Object] = {}
    for obj in source_ontology.objects:
        if obj.shape != Shape.POLYGON:
            continue

        # Find the bounding box object in the target ontology with the same name
        match = None
        for tobj in target_ontology.structure.objects:
            if tobj.shape != Shape.BOUNDING_BOX or tobj.name != obj.name:
                continue
            match = tobj
            break

        if match is None:
            match = target_ontology.structure.add_object(obj.name, Shape.BOUNDING_BOX)
        ontology_lookup[obj.name] = match

    target_ontology.save()
    return ontology_lookup


def convert_labels(keyfile_path: str, source_project_hash: str, target_project_hash: str):
    keyfile = Path(keyfile_path).expanduser().resolve()

    # create a connection
    user_client = EncordUserClient.create_with_ssh_private_key(keyfile.expanduser().read_text())

    # Initialize projects
    target_project = user_client.get_project(target_project_hash)
    source_project = user_client.get_project(source_project_hash)

    # Set up the ontology for the target project if not filled already.
    ontology_lookup = populate_ontology_of_target_project(user_client, source_project, target_project)
    target_project.refetch_ontology()

    # Initialize labels
    source_project_label_rows = initialize_label_rows(source_project)
    target_project_label_rows = initialize_label_rows(
        target_project,
        include_unlabeled=True,
    )

    # Lookup for reading labels from the source project
    source_project_label_rows_by_data_hash = {lr.data_hash: lr for lr in source_project_label_rows}

    bundle = target_project.create_bundle()
    bundle_size = 0

    # Perform the conversion from polygons to bounding boxes and upload.
    for target_lr in tqdm(target_project_label_rows, desc="Migrating labels"):
        source_lr = source_project_label_rows_by_data_hash.get(target_lr.data_hash)
        if source_lr is None:
            # No labels in source project for the given data point.
            continue

        should_save = False
        for obj in source_lr.get_object_instances():
            if obj.ontology_item.shape != Shape.POLYGON or obj.object_name not in ontology_lookup:
                continue

            new_instance = ontology_lookup[obj.object_name].create_instance()
            has_annotations = False
            for annotation in obj.get_annotations():
                has_annotations = True

                coords = cast(PolygonCoordinates, annotation.coordinates)
                xs, ys = zip(*((point.x, point.y) for point in coords.values))

                min_x, max_x = min(xs), max(xs)
                min_y, max_y = min(ys), max(ys)

                target_coordinates = BoundingBoxCoordinates(
                    height=max_y - min_y,
                    width=max_x - min_x,
                    top_left_x=min_x,
                    top_left_y=min_y,
                )
                new_instance.set_for_frames(coordinates=target_coordinates, frames=annotation.frame)

            if has_annotations:
                should_save = True
                target_lr.add_object_instance(new_instance)

        if should_save:
            bundle_size += 1
            target_lr.save(bundle=bundle)

        if bundle_size >= 200:
            try_execute(bundle.execute)
            bundle = target_project.create_bundle()
            bundle_size = 0

        try_execute(target_lr.save)

    if bundle_size > 0:
        try_execute(bundle.execute)

    print("Done!")


if __name__ == "__main__":
    keyfile_path = "/Users/chris-encord/ssh-private-key.txt"  # Replace with your keyfile path
    source_project_hash = "00000000-0000-0000-0000-000000000000"  # Replace with your source project hash
    target_project_hash = "00000000-0000-0000-0000-000000000000"  # Replace with your target project hash
    convert_labels(keyfile_path, source_project_hash, target_project_hash)



"""---
title: "Project"
slug: "sdk-ref-project"
hidden: false
metadata:
  title: "Project"
  description: "Encord SDK Project class"
category: "64e481b57b6027003f20aaa0"
---
"""

import datetime
from typing import Any, Dict, Iterable, Iterator, List, Optional, Set, Tuple, Union
from uuid import UUID

from encord.client import EncordClientProject
from encord.collection import ProjectCollection
from encord.common.deprecated import deprecated
from encord.common.utils import ensure_list, ensure_uuid_list
from encord.filter_preset import ProjectFilterPreset
from encord.http.bundle import Bundle
from encord.http.v2.api_client import ApiClient
from encord.objects import LabelRowV2, OntologyStructure
from encord.ontology import Ontology
from encord.orm.active import ActiveProjectMode
from encord.orm.analytics import (
    CollaboratorTimer,
    CollaboratorTimerParams,
    CollaboratorTimersGroupBy,
    TimeSpent,
    TimeSpentParams,
)
from encord.orm.cloud_integration import CloudIntegration
from encord.orm.collection import ProjectCollectionType
from encord.orm.dataset import Image, Video
from encord.orm.filter_preset import ActiveFilterPresetDefinition
from encord.orm.group import ProjectGroup
from encord.orm.label_log import LabelLog
from encord.orm.label_row import (
    AnnotationTaskStatus,
    LabelRow,
    LabelRowMetadata,
    LabelStatus,
    ShadowDataState,
)
from encord.orm.project import (
    CopyDatasetOptions,
    CopyLabelsOptions,
    ProjectDataset,
    ProjectDTO,
    ProjectStatus,
    ProjectType,
)
from encord.orm.project import Project as OrmProject
from encord.project_ontology.classification_type import ClassificationType
from encord.project_ontology.object_type import ObjectShape
from encord.project_ontology.ontology import Ontology as LegacyOntology
from encord.utilities.coco.datastructure import CategoryID, FrameIndex, ImageID
from encord.utilities.hash_utilities import convert_to_uuid
from encord.utilities.project_user import ProjectUser, ProjectUserRole
from encord.workflow import Workflow


class Project:
    """Access project related data and manipulate the project."""

    def __init__(
        self,
        client: EncordClientProject,
        project_instance: ProjectDTO,
        ontology: Optional[Ontology],
        api_client: ApiClient,
    ):
        self._client = client
        self._project_instance = project_instance
        self._ontology_internal = ontology
        self._api_client = api_client

        if project_instance.workflow:
            self._workflow = Workflow(api_client, project_instance.project_hash, project_instance.workflow)

    @property
    def project_hash(self) -> str:
        """Get the project hash (i.e. the Project ID)."""
        # Keeping the interface backward compatible, so converting UUID to str for now
        return str(self._project_instance.project_hash)

    @property
    def status(self) -> ProjectStatus:
        """Get the status of the project."""
        return self._project_instance.status

    @property
    def title(self) -> str:
        """Get the title of the project."""
        return self._project_instance.title

    @property
    def description(self) -> str:
        """Get the description of the project."""
        return self._project_instance.description

    @property
    def created_at(self) -> datetime.datetime:
        """Get the time the project was created at."""
        return self._project_instance.created_at

    @property
    def last_edited_at(self) -> datetime.datetime:
        """Get the time the project was last edited at."""
        return self._project_instance.last_edited_at

    @property
    @deprecated(version="0.1.95", alternative=".ontology_structure")
    def ontology(self) -> Dict[str, Any]:
        """Get the ontology of the project.

        DEPRECATED: Prefer using the :meth:`encord.Project.ontology_structure` method.
        This method returns the same structure as :meth:`encord.Project.ontology_structure`, just in
        raw python dictionary format.
        """
        return self._ontology.structure.to_dict()

    @property
    def ontology_hash(self) -> str:
        """Get the ontology hash of the project's ontology."""
        return self._project_instance.ontology_hash

    @property
    def ontology_structure(self) -> OntologyStructure:
        """Get the ontology structure of the project's ontology."""
        return self._ontology.structure

    @property
    def user_role(self) -> Optional[ProjectUserRole]:
        """Get the current user's role in the project.

        This may return `None` if the user is an organisational admin and has accessed the project e.g. using
        `include_org_access=True` of :meth:`encord.user_client.UserClient.list_projects`.
        """
        return self._project_instance.user_role

    @property
    def source_projects(self) -> Optional[List[str]]:
        """Get the source projects for a Training project. Returns None for non-Training projects."""
        return self._project_instance.source_projects

    @property
    @deprecated(version="0.1.117", alternative=".list_datasets")
    def datasets(self) -> List[Dict[str, Any]]:
        """DEPRECATED: Prefer using the :meth:`encord.project.list_datasets` class to work with the data.

        Get the info about datasets associated with this project.
        """
        return [project_dataset.to_dict(by_alias=False) for project_dataset in self.list_datasets()]

    @property
    def project_type(self) -> ProjectType:
        """Get the project type."""
        return self._project_instance.project_type

    @property
    @deprecated(version="0.1.104", alternative=".list_label_rows_v2")
    def label_rows(self) -> dict:
        """Get the label rows.
        DEPRECATED: Prefer using :meth:`list_label_rows_v2()` method and :meth:`LabelRowV2` class to work with the data.

        .. code::

            from encord.orm.label_row import LabelRowMetadata

            project = user_client.get_project("[project_hash]")

            label_rows = LabelRowMetadata.from_list(project.label_rows)

        """
        return self._client.get_project(include_labels_metadata=True).label_rows

    def refetch_data(self) -> None:
        """The Project class will only fetch its properties once. Use this function if you suspect the state of those
        properties to be dirty.
        """
        self._project_instance = self._client.get_project_v2()

    def refetch_ontology(self) -> None:
        """Update the ontology for the project to reflect changes on the backend."""
        self._ontology.refetch_data()

    def get_project(self) -> OrmProject:
        """This function is exposed for convenience. You are encouraged to use the property accessors instead."""
        return self._client.get_project()

    @property
    def workflow(self) -> Workflow:
        """Get the workflow of the project.

        Available only for workflow projects.
        """
        assert self.project_type == ProjectType.WORKFLOW, (
            "project.workflow property only available for workflow projects"
        )
        return self._workflow

    @property
    def _ontology(self) -> Ontology:
        if self._ontology_internal is None:
            self._ontology_internal = Ontology(
                Ontology._fetch_ontology(self._api_client, self.ontology_hash), self._api_client
            )  # lazy loading
        return self._ontology_internal

    def list_label_rows_v2(
        self,
        data_hashes: Optional[Union[List[str], List[UUID]]] = None,
        label_hashes: Optional[Union[List[str], List[UUID]]] = None,
        edited_before: Optional[Union[str, datetime.datetime]] = None,
        edited_after: Optional[Union[str, datetime.datetime]] = None,
        label_statuses: Optional[List[AnnotationTaskStatus]] = None,
        shadow_data_state: Optional[ShadowDataState] = None,
        data_title_eq: Optional[str] = None,
        data_title_like: Optional[str] = None,
        workflow_graph_node_title_eq: Optional[str] = None,
        workflow_graph_node_title_like: Optional[str] = None,
        include_workflow_graph_node: bool = True,
        include_client_metadata: bool = False,
        include_images_data: bool = False,
        include_children: bool = False,
        include_all_label_branches: bool = False,
        branch_name: Optional[str] = None,
    ) -> List[LabelRowV2]:
        """List label rows with various filtering options.

        Args:
            data_hashes: List of data hashes to filter by.
            label_hashes: List of label hashes to filter by.
            edited_before: Optionally filter to only rows last edited before the specified time.
            edited_after: Optionally filter to only rows last edited after the specified time.
            label_statuses: Optionally filter to only those label rows that have one of the specified :class:`~encord.orm.label_row.AnnotationTaskStatus`es.
            shadow_data_state: Optionally filter by data type in Benchmark QA projects. See :class:`~encord.orm.label_row.ShadowDataState`.
            data_title_eq: Optionally filter by exact title match.
            data_title_like: Optionally filter by fuzzy title match; SQL syntax.
            workflow_graph_node_title_eq: Optionally filter by exact match with workflow node title.
            workflow_graph_node_title_like: Optionally filter by fuzzy match with workflow node title; SQL syntax.
            include_workflow_graph_node: Include workflow graph node metadata in all the results. True by default.
            include_client_metadata: Optionally include client metadata into the result of this query.
            include_images_data: Optionally include image group metadata into the result of this query.
            include_children: Optionally include data group children rows into the result of this query.
            include_all_label_branches: Optionally include all label branches. They will be included as separate label row objects.
            branch_name: Optionally specify a branch name. A branch name cannot be specified if include_all_label_branches is set to True

        Returns:
            A list of :class:`~encord.objects.LabelRowV2` instances for all the matching label rows.
        """
        label_row_metadatas = self._client.list_label_rows(
            edited_before,
            edited_after,
            label_statuses,
            shadow_data_state,
            data_hashes=data_hashes,
            label_hashes=label_hashes,
            include_uninitialised_labels=True,
            include_children=include_children,
            data_title_eq=data_title_eq,
            data_title_like=data_title_like,
            workflow_graph_node_title_eq=workflow_graph_node_title_eq,
            workflow_graph_node_title_like=workflow_graph_node_title_like,
            include_workflow_graph_node=include_workflow_graph_node,
            include_client_metadata=include_client_metadata,
            include_images_data=include_images_data,
            include_all_label_branches=include_all_label_branches,
            branch_name=branch_name,
        )
        label_rows = [
            LabelRowV2(label_row_metadata, self._client, self._ontology) for label_row_metadata in label_row_metadatas
        ]
        return label_rows

    def add_users(self, user_emails: List[str], user_role: ProjectUserRole) -> List[ProjectUser]:
        """Add users to the project.

        If the user already exists in the Project, this operation succeeds but the `user_role` remains unchanged. The
        existing `user_role` is reflected in the `ProjectUser` instance returned.

        Args:
            user_emails: List of user emails to be added.
            user_role: The user role to assign to all users.

        Returns:
            List[ProjectUser]: A list of ProjectUser objects representing the added users.

        Raises:
            AuthorisationError: If the project API key is invalid.
            ResourceNotFoundError: If no project exists by the specified project EntityId.
            UnknownError: If an error occurs while adding the users to the project.
        """
        return self._client.add_users(user_emails, user_role)

    def list_users(self) -> Iterable[ProjectUser]:
        """List all users that have access to the project.

        Returns:
            Iterable[ProjectUser]: An iterable of ProjectUser objects.
        """
        yield from self._client.list_users(UUID(self.project_hash))

    def list_groups(self) -> Iterable[ProjectGroup]:
        """List all groups that have access to a particular project.

        Returns:
            Iterable[ProjectGroup]: An iterable of ProjectGroup objects.
        """
        project_hash = convert_to_uuid(self.project_hash)
        page = self._client.list_groups(project_hash)
        yield from page.results

    def add_group(self, group_hash: Union[List[UUID], UUID], user_role: ProjectUserRole):
        """Add a group to the project.

        Args:
            group_hash: List of group hashes or a single group hash to be added.
            user_role: User role that the group will be given.

        Returns:
            None
        """
        project_hash = convert_to_uuid(self.project_hash)
        if isinstance(group_hash, UUID):
            group_hash = [group_hash]
        self._client.add_groups(project_hash, group_hash, user_role)

    def remove_group(self, group_hash: Union[List[UUID], UUID]):
        """Remove a group from the project.

        Args:
            group_hash: List of group hashes or a single group hash to be removed.

        Returns:
            None
        """
        if isinstance(group_hash, UUID):
            group_hash = [group_hash]
        self._client.remove_groups(group_hash)

    def copy_project(
        self,
        copy_datasets: Union[bool, CopyDatasetOptions] = False,
        copy_collaborators=False,
        copy_models=False,
        *,
        copy_labels: Optional[CopyLabelsOptions] = None,
        new_title: Optional[str] = None,
        new_description: Optional[str] = None,
    ) -> str:
        """Copy the current project into a new one with copied contents including settings, datasets, and users.
        Labels and models are optional.

        Args:
            copy_datasets: If True, the datasets of the existing project are copied over, and new tasks are created from those datasets.
            copy_collaborators: If True, all users of the existing project are copied over with their current roles.
                                If label and/or annotator reviewer mapping is set, this will also be copied over.
            copy_models: If True, all models with their training information will be copied into the new project.
            copy_labels: Options for copying labels, defined in `CopyLabelsOptions`.
            new_title: When provided, will be used as the title for the new project.
            new_description: When provided, will be used as the description for the new project.

        Returns:
            str: The EntityId of the newly created project.

        Raises:
            AuthorisationError: If the project API key is invalid.
            ResourceNotFoundError: If no project exists by the specified project EntityId.
            UnknownError: If an error occurs while copying the project.
        """
        return self._client.copy_project(
            new_title=new_title,
            new_description=new_description,
            copy_datasets=copy_datasets,
            copy_collaborators=copy_collaborators,
            copy_models=copy_models,
            copy_labels=copy_labels,
        )

    def add_datasets(self, dataset_hashes: List[str]) -> bool:
        """Add datasets to the project.

        Args:
            dataset_hashes: List of dataset hashes of the datasets to be added.

        Returns:
            bool: True if the datasets were successfully added, False otherwise.

        Raises:
            AuthenticationError: If the project API key is invalid.
            AuthorisationError: If access to the specified resource is restricted.
            ResourceNotFoundError: If one or more datasets don't exist by the specified dataset_hashes.
            UnknownError: If an error occurs while adding the datasets to the project.
            OperationNotAllowed: If the write operation is not allowed by the API key.
        """
        res = self._client.add_datasets(dataset_hashes)
        self.refetch_data()
        return res

    def remove_datasets(self, dataset_hashes: List[str]) -> bool:
        """Remove datasets from the project.

        Args:
            dataset_hashes: List of dataset hashes of the datasets to be removed.

        Returns:
            bool: True if the datasets were successfully removed, False otherwise.

        Raises:
            AuthenticationError: If the project API key is invalid.
            AuthorisationError: If access to the specified resource is restricted.
            ResourceNotFoundError: If no dataset exists by the specified dataset_hash (uid).
            UnknownError: If an error occurs while removing the datasets from the project.
            OperationNotAllowed: If the operation is not allowed by the API key.
        """
        res = self._client.remove_datasets(dataset_hashes)
        self.refetch_data()
        return res

    @deprecated(version="0.1.95", alternative=".ontology_structure")
    def get_project_ontology(self) -> LegacyOntology:
        """DEPRECATED: Prefer using the `ontology_structure` property accessor instead.

        Returns:
            LegacyOntology: The project's ontology.
        """
        return self._client.get_project_ontology()

    @deprecated("0.1.102", alternative="encord.ontology.Ontology class")
    def add_object(self, name: str, shape: ObjectShape) -> bool:
        """DEPRECATED: Prefer using :class:`Ontology [encord.ontology.Ontology]` to manipulate ontology.

        Add an object to an ontology.

        ATTENTION: This legacy method will affect all the projects sharing the same ontology.

        Args:
            name: The name of the object.
            shape: The shape of the object. (BOUNDING_BOX, POLYGON, POLYLINE, or KEY_POINT)

        Returns:
            bool: True if the object was added successfully, False otherwise.

        Raises:
            AuthenticationError: If the project API key is invalid.
            AuthorisationError: If access to the specified resource is restricted.
            UnknownError: If an error occurs while adding the object to the project ontology.
            OperationNotAllowed: If the operation is not allowed by the API key.
            ValueError: If invalid arguments are supplied in the function call.
        """
        res = self._client.add_object(name, shape)
        self.refetch_ontology()
        return res

    @deprecated("0.1.102", alternative="encord.ontology.Ontology class")
    def add_classification(
        self,
        name: str,
        classification_type: ClassificationType,
        required: bool,
        options: Optional[Iterable[str]] = None,
    ):
        """DEPRECATED: Prefer using :class:`Ontology encord.ontology.Ontology` to manipulate ontology.

        Add a classification to an ontology.

        ATTENTION: This legacy method will affect all the projects sharing the same ontology.

        Args:
            name: The name of the classification.
            classification_type: The classification type (RADIO, TEXT, or CHECKLIST).
            required: Whether this classification is required by the annotator.
            options: The list of options for the classification (to be set to None for texts).

        Raises:
            AuthenticationError: If the project API key is invalid.
            AuthorisationError: If access to the specified resource is restricted.
            UnknownError: If an error occurs while adding the classification to the project ontology.
            OperationNotAllowed: If the operation is not allowed by the API key.
            ValueError: If invalid arguments are supplied in the function call.
        """
        res = self._client.add_classification(name, classification_type, required, options)
        self.refetch_ontology()
        return res

    def object_interpolation(
        self,
        key_frames,
        objects_to_interpolate,
    ):
        """Run object interpolation algorithm on project labels (requires an editor ontology and feature uids).

        Interpolation is supported for bounding box, polygon, and keypoint.

        Args:
            key_frames: Labels for frames to be interpolated. Key frames are consumed in the form::

            ```python
                {
                    "[frame_number]": {
                        "objects": [
                            {
                                "objectHash": "[object_hash]",
                                "featureHash": "[feature_hash]",
                                "polygon": {
                                    "0": { "x": x1, "y": y1, },
                                    "1": { "x": x2, "y": y2, },
                                    # ...,
                                }
                            },
                            # ...
                        ]
                    },
                    # ...,
                }
            ```

            objects_to_interpolate: List of object uids (hashes) of objects to interpolate.

        Returns:
            dict: Full set of filled frames including interpolated objects.

        Raises:
            AuthenticationError: If the project API key is invalid.
            AuthorisationError: If access to the specified resource is restricted.
            UnknownError: If an error occurs while running interpolation.
        """
        return self._client.object_interpolation(key_frames, objects_to_interpolate)

    def get_data(self, data_hash: str, get_signed_url: bool = False) -> Tuple[Optional[Video], Optional[List[Image]]]:
        """Retrieve information about a video or image group.

        Args:
            data_hash: The uid of the data object.
            get_signed_url: Optionally return signed URLs for timed public access to that resource (default False).

        Returns:
            A tuple consisting of the video (if it exists) and a list of individual images (if they exist).

        Raises:
            AuthenticationError: If the project API key is invalid.
            AuthorisationError: If access to the specified resource is restricted.
            UnknownError: If an error occurs while retrieving the object.
        """
        return self._client.get_data(data_hash, get_signed_url)

    def get_label_logs(
        self,
        user_hash: Optional[str] = None,
        data_hash: Optional[str] = None,
        from_unix_seconds: Optional[int] = None,
        to_unix_seconds: Optional[int] = None,
        after: Optional[datetime.datetime] = None,
        before: Optional[datetime.datetime] = None,
        user_email: Optional[str] = None,
    ) -> List[LabelLog]:
        """Get label logs, which represent the actions taken in the UI to create labels.

        All arguments can be left as `None` if no filtering should be applied.

        Args:
            user_hash: Filter the label logs by the user.
            data_hash: Filter the label logs by the data_hash.
            from_unix_seconds: Filter the label logs to only include labels after this timestamp. **Deprecated**: use parameter **after** instead.
            to_unix_seconds: Filter the label logs to only include labels before this timestamp. **Deprecated**: use parameter **before** instead.
            after: Filter the label logs to only include labels after the specified time.
            before: Filter the label logs to only include labels before the specified time.
            user_email: Filter by the annotator email.

        Returns:
            List of label logs.
        """
        return self._client.get_label_logs(
            user_hash,
            data_hash,
            from_unix_seconds,
            to_unix_seconds,
            after,
            before,
            user_email,
        )

    @deprecated(version="0.1.154", alternative="EncordUserClient.get_cloud_integrations")
    def get_cloud_integrations(self) -> List[CloudIntegration]:
        """Get the list of cloud integrations.

        Returns:
            List of CloudIntegration objects.
        """
        return self._client.get_cloud_integrations()

    @deprecated(version="0.1.104", alternative=".list_label_rows_v2")
    def list_label_rows(
        self,
        edited_before: Optional[Union[str, datetime.datetime]] = None,
        edited_after: Optional[Union[str, datetime.datetime]] = None,
        label_statuses: Optional[List[AnnotationTaskStatus]] = None,
        shadow_data_state: Optional[ShadowDataState] = None,
        *,
        include_uninitialised_labels=False,
        label_hashes: Optional[List[str]] = None,
        data_hashes: Optional[List[str]] = None,
    ) -> List[LabelRowMetadata]:
        """DEPRECATED - use `list_label_rows_v2` to manage label rows instead.

        Args:
            edited_before: Optionally filter to only rows last edited before the specified time.
            edited_after: Optionally filter to only rows last edited after the specified time.
            label_statuses: Optionally filter to only those label rows that have one of the specified AnnotationTaskStatus.
            shadow_data_state: Optionally filter by data type in Benchmark QA projects. See ShadowDataState.
            include_uninitialised_labels: Whether to return only label rows that are "created" and have a label_hash (default).
                If set to `True`, this will return all label rows, including those that do not have a label_hash.
            data_hashes: List of data hashes to filter by.
            label_hashes: List of label hashes to filter by.

        Returns:
            A list of LabelRowMetadata instances for all the matching label rows.

        Raises:
            UnknownError: If an error occurs while retrieving the data.
        """
        return self._client.list_label_rows(
            edited_before,
            edited_after,
            label_statuses,
            shadow_data_state,
            include_uninitialised_labels=include_uninitialised_labels,
            label_hashes=label_hashes,
            data_hashes=data_hashes,
        )

    @deprecated(version="0.1.123", alternative=".list_label_rows_v2")
    def get_label_row(
        self,
        uid: str,
        get_signed_url: bool = True,
        *,
        include_object_feature_hashes: Optional[Set[str]] = None,
        include_classification_feature_hashes: Optional[Set[str]] = None,
        include_reviews: bool = False,
    ) -> LabelRow:
        """DEPRECATED: Prefer using the list_label_rows_v2 function to interact with label rows.

        Retrieve label row. If you need to retrieve multiple label rows, prefer using get_label_rows instead.

        Args:
            uid: A label_hash (uid) string.
            get_signed_url: Whether to generate signed urls to the data asset. Generating these should be disabled if the signed urls are not used to speed up the request.
            include_object_feature_hashes: If None all the objects will be included. Otherwise, only objects labels will be included of which the feature_hash has been added.
            include_classification_feature_hashes: If None all the classifications will be included. Otherwise, only classification labels will be included of which the feature_hash has been added.
            include_reviews: Whether to request read only information about the reviews of the label row.

        Returns:
            LabelRow: A label row instance.

        Raises:
            AuthenticationError: If the project API key is invalid.
            AuthorisationError: If access to the specified resource is restricted.
            ResourceNotFoundError: If no label exists by the specified label_hash (uid).
            UnknownError: If an error occurs while retrieving the label.
            OperationNotAllowed: If the read operation is not allowed by the API key.
        """
        return self._client.get_label_row(
            uid,
            get_signed_url,
            include_object_feature_hashes=include_object_feature_hashes,
            include_classification_feature_hashes=include_classification_feature_hashes,
            include_reviews=include_reviews,
        )

    @deprecated(version="0.1.123", alternative=".list_label_rows_v2")
    def get_label_rows(
        self,
        uids: List[str],
        get_signed_url: bool = True,
        *,
        include_object_feature_hashes: Optional[Set[str]] = None,
        include_classification_feature_hashes: Optional[Set[str]] = None,
        include_reviews: bool = False,
    ) -> List[LabelRow]:
        """DEPRECATED: Prefer using the list_label_rows_v2 function to interact with label rows.

        Retrieve a list of label rows. Duplicates will be dropped. The result will come back in a random order.

        Args:
            uids: A list of label_hash (uid).
            get_signed_url: Whether to generate signed urls to the data asset. Generating these should be disabled if the signed urls are not used to speed up the request.
            include_object_feature_hashes: If None all the objects will be included. Otherwise, only objects labels will be included of which the feature_hash has been added.
            include_classification_feature_hashes: If None all the classifications will be included. Otherwise, only classification labels will be included of which the feature_hash has been added.
            include_reviews: Whether to request read only information about the reviews of the label row.

        Returns:
            List of LabelRow instances.

        Raises:
            MultiLabelLimitError: If too many labels were requested. Check the error's maximum_labels_allowed field to read the most up to date error limit.
            AuthenticationError: If the project API key is invalid.
            AuthorisationError: If access to the specified resource is restricted.
            ResourceNotFoundError: If no label exists by the specified label_hash (uid).
            UnknownError: If an error occurs while retrieving the label.
            OperationNotAllowed: If the read operation is not allowed by the API key.
        """
        return self._client.get_label_rows(
            uids,
            get_signed_url,
            include_object_feature_hashes=include_object_feature_hashes,
            include_classification_feature_hashes=include_classification_feature_hashes,
            include_reviews=include_reviews,
        )

    @deprecated(version="0.1.123", alternative=".list_label_rows_v2")
    def save_label_row(self, uid, label, validate_before_saving: bool = False):
        """DEPRECATED: Prefer using the list_label_rows_v2 function to interact with label rows.

        Save an existing label row.

        If you have a series of frame labels and have not updated answer
        dictionaries, call the construct_answer_dictionaries utility function
        to do so prior to saving labels.

        Args:
            uid: A label_hash (uid) string.
            label: A label row instance.
            validate_before_saving: Enable stricter server-side integrity checks. Boolean, `False` by default.

        Returns:
            bool: True if the label row is successfully saved, False otherwise.

        Raises:
            AuthenticationError: If the project API key is invalid.
            AuthorisationError: If access to the specified resource is restricted.
            ResourceNotFoundError: If no label exists by the specified label_hash (uid).
            UnknownError: If an error occurs while saving the label.
            OperationNotAllowed: If the write operation is not allowed by the API key.
            AnswerDictionaryError: If an object or classification instance is missing in answer dictionaries.
            CorruptedLabelError: If a blurb is corrupted (e.g., if the frame labels have more frames than the video).
        """
        return self._client.save_label_row(uid, label, validate_before_saving)

    @deprecated(version="0.1.123", alternative=".list_label_rows_v2")
    def create_label_row(self, uid: str):
        """DEPRECATED: Prefer using the list_label_rows_v2 function to interact with label rows.

        Create a label row (for data in a project not previously labeled).

        Args:
            uid: The data_hash (uid) of the data unit being labeled.
                Available in client.get_project().get('label_rows')
                where label_status is NOT_LABELLED.

        Returns:
            LabelRow: A label row instance.

        Raises:
            AuthenticationError: If the project API key is invalid.
            AuthorisationError: If access to the specified resource is restricted.
            UnknownError: If an error occurs while saving the label.
            OperationNotAllowed: If the write operation is not allowed by the API key.
            AnswerDictionaryError: If an object or classification instance is missing in answer dictionaries.
            CorruptedLabelError: If a blurb is corrupted (e.g., if the frame labels have more frames than the video).
            ResourceExistsError: If a label row already exists for this project data. Avoids overriding existing work.
        """
        return self._client.create_label_row(uid)

    def create_bundle(self, bundle_size: Optional[int] = None) -> Bundle:
        """Initializes a bundle to reduce the number of network calls performed by the Encord SDK.

        See the :class:`encord.http.bundle.Bundle` documentation for more details.

        Args:
            bundle_size: maximum number of items bundled. If more actions provided to the bundle, they will be
                automatically split into separate api calls.

        Returns:
            Bundle: An instance of the Bundle class.
        """
        return Bundle(bundle_size=bundle_size)

    @deprecated(version="0.1.157", alternative=".list_time_spent")
    def list_collaborator_timers(
        self,
        after: datetime.datetime,
        before: Optional[datetime.datetime] = None,
        group_by_data_unit: bool = True,
    ) -> Iterable[CollaboratorTimer]:
        """**DEPRECATED** - Use `list_time_spent`. `list_time_spent` provides more comprehensive and accurate information.

        Provides information about time spent by each collaborator who has worked on the project within a specified range of dates.

        This endpoint is deprecated and retrieves collaborator timers from the Legacy Performance Dashboards, not the Upgraded Analytics Dashboard.

        Args:
            after: The beginning of the period of interest.
            before: The end of the period of interest.
            group_by_data_unit: If True, time spent by a collaborator for each data unit is provided separately.
                                If False, all time spent in the scope of the project is aggregated together.

        Yields:
            CollaboratorTimer: Information about the time spent by each collaborator.
        """
        params = CollaboratorTimerParams(
            project_hash=self.project_hash,
            after=after,
            before=before,
            group_by=(CollaboratorTimersGroupBy.DATA_UNIT if group_by_data_unit else CollaboratorTimersGroupBy.PROJECT),
            page_size=100,
        )

        yield from self._client.get_collaborator_timers(params)

    def list_time_spent(
        self,
        start: datetime.datetime,
        end: Optional[datetime.datetime] = None,
        workflow_stage_uuid: Union[List[UUID], List[str], UUID, str, None] = None,
        user_email: Union[List[str], str, None] = None,
    ) -> Iterable[TimeSpent]:
        """
        Get time spent by collaborators on a task per day. If a task spans multiple days, the time spent on each day will be returned separately.

        Args:
            start: The beginning of the period of interest.
            end: The end of the period of interest.
            workflow_stage_uuid: The UUID or list of UUIDs of the workflow stages of interest.
            user_email: The email or list of emails of the users of interest.

        Returns:
            Iterable[TimeSpent]: Information about the time spent by a collaborator on a task per day.
        """
        params = TimeSpentParams(
            project_uuid=self.project_hash,
            after=start,
            before=end,
            workflow_stage_uuids=ensure_uuid_list(workflow_stage_uuid),
            user_emails=ensure_list(user_email),
        )

        yield from self._client.get_time_spent(params)

    def list_datasets(self) -> Iterable[ProjectDataset]:
        """List all datasets associated with the project.

        Returns:
            Iterable[ProjectDataset]: An iterable of ProjectDataset instances.
        """
        return self._client.list_project_datasets(self._project_instance.project_hash)

    def import_coco_labels(
        self,
        labels_dict: Dict[str, Any],
        category_id_to_feature_hash: Dict[CategoryID, str],
        image_id_to_frame_index: Dict[ImageID, FrameIndex],
        branch_name: Optional[str] = None,
        confidence_field_name: Optional[str] = None,
    ) -> None:
        """Import labels in COCO format to an Encord Project.

        Args:
            labels_dict (Dict[str, Any]): A dictionary in COCO annotation format.
            category_id_to_feature_hash (Dict[CategoryID, str]): A mapping of category IDs from the COCO data to their corresponding feature hashes in the Project's Ontology.
            image_id_to_frame_index (Dict[ImageID, FrameIndex]): A mapping of image IDs to FrameIndex(data_hash, frame_offset), used to locate the corresponding frames in the Encord Project.
            branch_name (Optional[str]): Optionally specify a branch name. Defaults to the `main` branch.
            confidence_field_name (Optional[str]): Optionally specify the name of the confidence field in the COCO annotations. Defaults to assigning `1.0` as confidence value to all annotations.
        """
        from encord.utilities.coco.datastructure import CocoRootModel
        from encord.utilities.coco.importer import import_coco_labels

        coco = CocoRootModel.from_dict(labels_dict)
        import_coco_labels(
            self,
            coco,
            category_id_to_feature_hash,
            image_id_to_frame_index,
            branch_name=branch_name,
            confidence_field_name=confidence_field_name,
        )

    def export_coco_labels(
        self,
        label_hashes: Optional[List[str]] = None,
        include_object_feature_hashes: Optional[Set[str]] = None,
        include_classification_feature_hashes: Optional[Set[str]] = None,
        branch_name: Optional[str] = None,
    ) -> Dict[str, Any]:
        """Export labels from the project to the COCO format.
        This method requires the 'coco' extra to be installed. Install it using:
        `pip install encord[coco]`.

        Args:
            label_hashes: List of label hashes to include. If not provided, all label rows will be included.
            include_object_feature_hashes: If `None`, all objects will be included.
                 Otherwise, only objects with the specified feature hashes will be included.
            include_classification_feature_hashes: If `None`, all classifications will be included.
                Otherwise, only classifications with the specified feature hashes will be included.
            branch_name: Optionally specify a branch name. Defaults to the `main` branch.

        Returns:
            Dict[str, Any]: A dictionary in the COCO format containing the exported labels,
                including annotations and metadata conforming to COCO standards.
                The dictionary also includes additional fields specific to Encord,
                providing supplementary information not defined in the COCO standard.

        Raises:
            ImportError: If the 'coco' extra dependencies are not installed.
        """
        from encord.utilities.coco.exporter import CocoExporter

        label_rows = self.list_label_rows_v2(label_hashes=label_hashes, branch_name=branch_name)
        with self.create_bundle() as bundle:
            for row in label_rows:
                row.initialise_labels(
                    include_object_feature_hashes=include_object_feature_hashes,
                    include_classification_feature_hashes=include_classification_feature_hashes,
                    bundle=bundle,
                )
        labels = [row.to_encord_dict() for row in label_rows]
        coco_labels = CocoExporter(labels, ontology=self.ontology_structure).export()
        return coco_labels

    def get_collection(self, collection_uuid: Union[str, UUID]) -> ProjectCollection:
        return ProjectCollection._get_collection(
            project_client=self._client,
            ontology=self._ontology,
            project_uuid=self._project_instance.project_hash,
            collection_uuid=UUID(collection_uuid) if isinstance(collection_uuid, str) else collection_uuid,
        )

    def list_collections(
        self,
        collection_uuids: Optional[List[Union[str, UUID]]] = None,
        page_size: Optional[int] = None,
    ) -> Iterator[ProjectCollection]:
        """List all collections associated to the project.

        Args:
            collection_uuids: The unique identifiers (UUIDs) of the collections to retrieve.
            page_size (int): Number of items to return per page.  Default if not specified is 100. Maximum value is 1000.

        Returns:
            The list of collections which match the given criteria.

        Raises:
            ValueError: If any of the collection uuids is a badly formed UUID.
            :class:`encord.exceptions.AuthorizationError` : If the user does not have access to it.
        """
        collections = (
            [UUID(collection) if isinstance(collection, str) else collection for collection in collection_uuids]
            if collection_uuids is not None
            else None
        )
        return ProjectCollection._list_collections(
            project_client=self._client,
            ontology=self._ontology,
            project_uuid=self._project_instance.project_hash,
            collection_uuids=collections,
            page_size=page_size,
        )

    def delete_collection(self, collection_uuid: Union[str, UUID]) -> None:
        """Delete a project collection by its UUID if it exists.

        Args:
            collection_uuid: The unique identifier (UUID) of the collection to delete.

        Returns:
            None
        Raises:
            ValueError: If `collection_uuid` is a badly formed UUID.
            :class:`encord.exceptions.AuthorizationError` : If the user does not have access to it.
        """
        if isinstance(collection_uuid, str):
            collection_uuid = UUID(collection_uuid)
        ProjectCollection._delete_collection(
            self._client._api_client, self._project_instance.project_hash, collection_uuid
        )

    def create_collection(
        self, name: str, description: str = "", collection_type: ProjectCollectionType = ProjectCollectionType.FRAME
    ) -> ProjectCollection:
        """Create a project collection.

        Args:
            name: The name of the collection.
            description: The description of the collection.
            collection_type: The type of the collection, could be either frame or label.

        Returns:
            ProjectCollection: Newly created collection.

        Raises:
            :class:`encord.exceptions.AuthorizationError` : If the user does not have access to the folder.
        """
        new_uuid = ProjectCollection._create_collection(
            self._client._api_client, self._project_instance.project_hash, name, description, collection_type
        )
        return self.get_collection(new_uuid)

    def active_sync(self) -> None:
        """Sync the associated Active project"""
        self._client.active_sync()

    def active_import(self, project_mode: ActiveProjectMode, *, video_sampling_rate: Optional[float] = None) -> None:
        """Import the associated Active project. Progress in the app
        Args:
            project_mode: Active projects can be imported up to a certain stage. Use the ActiveProjectModeEnum to select the stage
            video_sampling_rate: Optional[float]: For videos, what's the sampling rate of frames for analysis
        Returns:
            None
        """
        self._client.active_import(project_mode, video_sampling_rate)

    def list_filter_presets(
        self,
        filter_preset_uuids: Optional[List[Union[str, UUID]]] = None,
        page_size: Optional[int] = None,
    ) -> Iterator[ProjectFilterPreset]:
        """List all filter presets associated to the project.

        Args:
            filter_preset_uuids: The unique identifiers (UUIDs) of the filter presets to retrieve.
            page_size (int): Number of items to return per page.  Default if not specified is 100. Maximum value is 1000.

        Returns:
            The list of filter presets which match the given criteria.

        Raises:
            ValueError: If any of the filter preset uuids is a badly formed UUID.
            :class:`encord.exceptions.AuthorizationError` : If the user does not have access to it.
        """
        filter_presets = (
            [
                UUID(filter_preset) if isinstance(filter_preset, str) else filter_preset
                for filter_preset in filter_preset_uuids
            ]
            if filter_preset_uuids is not None
            else None
        )
        return ProjectFilterPreset._list_filter_presets(
            client=self._client._api_client,
            project_uuid=self._project_instance.project_hash,
            filter_preset_uuids=filter_presets,
            page_size=page_size,
        )

    def get_filter_preset(self, filter_preset_uuid: Union[str, UUID]) -> ProjectFilterPreset:
        return ProjectFilterPreset._get_filter_preset(
            client=self._client._api_client,
            project_uuid=self._project_instance.project_hash,
            filter_preset_uuid=UUID(filter_preset_uuid) if isinstance(filter_preset_uuid, str) else filter_preset_uuid,
        )

    def delete_filter_preset(self, filter_preset_uuid: Union[str, UUID]) -> None:
        ProjectFilterPreset._delete_filter_preset(
            client=self._client._api_client,
            project_uuid=self._project_instance.project_hash,
            filter_preset_uuid=UUID(filter_preset_uuid) if isinstance(filter_preset_uuid, str) else filter_preset_uuid,
        )

    def create_filter_preset(self, name: str, filter_preset: ActiveFilterPresetDefinition) -> ProjectFilterPreset:
        uuid = ProjectFilterPreset._create_filter_preset(
            client=self._client._api_client,
            project_uuid=self._project_instance.project_hash,
            name=name,
            filter_preset=filter_preset,
        )
        return ProjectFilterPreset._get_filter_preset(
            client=self._client._api_client,
            project_uuid=self._project_instance.project_hash,
            filter_preset_uuid=uuid,
        )

    def set_status(self, status: ProjectStatus):
        """Set the status of the project.

        Args:
            status: The new status of the project.
        """
        self._client.set_status(status)



from encord.objects import (
    ChecklistAttribute,
    FlatOption,
    NestableOption,
    Object,
    OntologyStructure,
    RadioAttribute,
    TextAttribute,
)
from encord.objects.common import Shape
from encord.objects.ontology_labels_impl import Classification

all_types_structure = OntologyStructure(
    objects=[
        Object(
            uid=1, name="Box", color="#D33115", shape=Shape.BOUNDING_BOX, feature_node_hash="MjI2NzEy", attributes=[]
        ),
        Object(
            uid=2, name="Polygon", color="#E27300", shape=Shape.POLYGON, feature_node_hash="ODkxMzAx", attributes=[]
        ),
        Object(
            uid=3, name="Polyline", color="#16406C", shape=Shape.POLYLINE, feature_node_hash="OTcxMzIy", attributes=[]
        ),
        Object(
            uid=4,
            name="Keypoint Dynamic Answers",
            color="#FE9200",
            shape=Shape.POINT,
            feature_node_hash="MTY2MTQx",
            attributes=[
                TextAttribute(
                    uid=[5, 1], feature_node_hash="OTkxMjU1", name="First name", required=False, dynamic=True
                ),
                ChecklistAttribute(
                    uid=[5, 2],
                    feature_node_hash="ODcxMDAy",
                    name="Mood",
                    required=False,
                    dynamic=True,
                    options=[
                        FlatOption(uid=[5, 2, 1], feature_node_hash="MTE5MjQ3", label="Angry", value="angry"),
                        FlatOption(uid=[5, 2, 2], feature_node_hash="Nzg3MDE3", label="Sad", value="sad"),
                    ],
                ),
                RadioAttribute(
                    uid=[6, 1],
                    feature_node_hash="MTExM9I3",
                    name="Radio level 1 ",
                    required=False,
                    dynamic=True,
                    options=[
                        NestableOption(
                            uid=[6, 1, 1],
                            feature_node_hash="MT9xNDQ5",
                            label="1 option 1",
                            value="1_option_1",
                            nested_options=[
                                TextAttribute(
                                    uid=[6, 1, 1, 1],
                                    feature_node_hash="MjE2O9E0",
                                    name="1 1 text",
                                    required=False,
                                    dynamic=False,
                                )
                            ],
                        ),
                        NestableOption(
                            uid=[6, 1, 2],
                            feature_node_hash="9TcxMjAy",
                            label="1 option 2",
                            value="1_option_2",
                            nested_options=[
                                RadioAttribute(
                                    uid=[6, 1, 2, 1],
                                    feature_node_hash="NDYyM9Qx",
                                    name="1 2 radio 1",
                                    required=False,
                                    dynamic=False,
                                    options=[
                                        NestableOption(
                                            uid=[6, 1, 2, 1, 1],
                                            feature_node_hash="MTY0M9U2",
                                            label="1 2 1 option 1",
                                            value="1_2_1_option_1",
                                            nested_options=[],
                                        ),
                                        NestableOption(
                                            uid=[6, 1, 2, 1, 2],
                                            feature_node_hash="MTI49jQy",
                                            label="1 2 1 option ",
                                            value="1_2_1_option_",
                                            nested_options=[],
                                        ),
                                    ],
                                )
                            ],
                        ),
                    ],
                ),
            ],
        ),
        Object(
            uid=5,
            name="Nested Box",
            color="#FCDC00",
            shape=Shape.BOUNDING_BOX,
            feature_node_hash="MTA2MjAx",
            attributes=[
                TextAttribute(
                    uid=[5, 1], feature_node_hash="OTkxMjU1", name="First name", required=False, dynamic=False
                ),
                ChecklistAttribute(
                    uid=[5, 2],
                    feature_node_hash="ODcxMDAy",
                    name="Mood",
                    required=False,
                    dynamic=False,
                    options=[
                        FlatOption(uid=[5, 2, 1], feature_node_hash="MTE5MjQ3", label="Angry", value="angry"),
                        FlatOption(uid=[5, 2, 2], feature_node_hash="Nzg3MDE3", label="Sad", value="sad"),
                    ],
                ),
            ],
        ),
        Object(
            uid=6,
            name="Deeply Nested Polygon",
            color="#DBDF00",
            shape=Shape.POLYGON,
            feature_node_hash="MTM1MTQy",
            attributes=[
                TextAttribute(
                    uid=[5, 1], feature_node_hash="OTk555U1", name="First name", required=False, dynamic=False
                ),
                ChecklistAttribute(
                    uid=[5, 2],
                    feature_node_hash="ODc555Ay",
                    name="Mood",
                    required=False,
                    dynamic=False,
                    options=[
                        FlatOption(uid=[5, 2, 1], feature_node_hash="MT5555Q3", label="Angry", value="angry"),
                        FlatOption(uid=[5, 2, 2], feature_node_hash="Nzg5555E3", label="Sad", value="sad"),
                    ],
                ),
                RadioAttribute(
                    uid=[6, 1],
                    feature_node_hash="MTExMjI3",
                    name="Radio level 1 ",
                    required=False,
                    dynamic=False,
                    options=[
                        NestableOption(
                            uid=[6, 1, 1],
                            feature_node_hash="MTExNDQ5",
                            label="1 option 1",
                            value="1_option_1",
                            nested_options=[
                                TextAttribute(
                                    uid=[6, 1, 1, 1],
                                    feature_node_hash="MjE2OTE0",
                                    name="1 1 text",
                                    required=False,
                                    dynamic=False,
                                )
                            ],
                        ),
                        NestableOption(
                            uid=[6, 1, 2],
                            feature_node_hash="MTcxMjAy",
                            label="1 option 2",
                            value="1_option_2",
                            nested_options=[
                                ChecklistAttribute(
                                    uid=[5, 2],
                                    feature_node_hash="ODc666Ay",
                                    name="Mood",
                                    required=False,
                                    dynamic=False,
                                    options=[
                                        FlatOption(
                                            uid=[5, 2, 1], feature_node_hash="MT66665Q3", label="Angry", value="angry"
                                        ),
                                        FlatOption(
                                            uid=[5, 2, 2], feature_node_hash="Nzg66665E3", label="Sad", value="sad"
                                        ),
                                    ],
                                )
                            ],
                        ),
                    ],
                ),
            ],
        ),
        Object(
            uid=7,
            name="cone object",
            color="#A4DD00",
            shape=Shape.SKELETON,
            feature_node_hash="MTczNjQx",
            attributes=[],
        ),
        Object(
            uid=8,
            name="audio object",
            color="#A4FF00",
            shape=Shape.AUDIO,
            feature_node_hash="KVfzNkFy",
            attributes=[],
        ),
        Object(
            uid=9,
            name="text object",
            color="#A4FF00",
            shape=Shape.TEXT,
            feature_node_hash="textFeatureNodeHash",
            attributes=[],
        ),
    ],
    classifications=[
        Classification(
            uid=1,
            feature_node_hash="NzIxNTU1",
            attributes=[
                RadioAttribute(
                    uid=[1, 1],
                    feature_node_hash="MjI5MTA5",
                    name="Radio classification 1",
                    required=False,
                    dynamic=False,
                    options=[
                        NestableOption(
                            uid=[1, 1, 1],
                            feature_node_hash="MTcwMjM5",
                            label="cl 1 option 1",
                            value="cl_1_option_1",
                            nested_options=[],
                        ),
                        NestableOption(
                            uid=[1, 1, 2],
                            feature_node_hash="MjUzMTg1",
                            label="cl 1 option 2",
                            value="cl_1_option_2",
                            nested_options=[
                                TextAttribute(
                                    uid=[1, 1, 2, 1],
                                    feature_node_hash="MTg0MjIw",
                                    name="cl 1 2 text",
                                    required=False,
                                    dynamic=False,
                                )
                            ],
                        ),
                    ],
                )
            ],
        ),
        Classification(
            uid=2,
            feature_node_hash="jPOcEsbw",
            attributes=[
                TextAttribute(
                    uid=[2, 1],
                    feature_node_hash="OxrtEM+v",
                    name="Text classification",
                    required=False,
                    dynamic=False,
                )
            ],
        ),
        Classification(
            uid=3,
            feature_node_hash="3DuQbFxo",
            attributes=[
                ChecklistAttribute(
                    uid=[3, 1],
                    feature_node_hash="9mwWr3OE",
                    name="Checklist classification",
                    required=False,
                    dynamic=False,
                    options=[
                        FlatOption(
                            uid=[3, 1, 1],
                            feature_node_hash="fvLjF0qZ",
                            label="Checklist classification answer 1",
                            value="checklist_classification_answer_1",
                        ),
                        FlatOption(
                            uid=[3, 1, 2],
                            feature_node_hash="a4r7nK9i",
                            label="Checklist classification answer 2",
                            value="checklist_classification_answer_2",
                        ),
                    ],
                )
            ],
        ),
    ],
)



from typing import Any, Dict
from unittest.mock import patch

import pytest

from encord.objects.ontology_structure import OntologyStructure
from tests.objects.data.data_1 import labels as BASE_LABEL_DICT
from tests.objects.data.data_1 import ontology as BASE_ONTOLOGY_DICT

ontology_structure = OntologyStructure.from_dict(BASE_ONTOLOGY_DICT)
EXPECTED_COCO_RESULT: Dict[str, Any] = {
    "info": {
        "description": "failing_video_new.mp4",
        "contributor": None,
        "date_created": None,
        "url": None,
        "version": None,
        "year": None,
    },
    "categories": [
        {"supercategory": "bounding_box", "id": 1, "name": "Epiglottis "},
        {"supercategory": "bounding_box", "id": 2, "name": "Larynx"},
        {"supercategory": "bounding_box", "id": 3, "name": "Oesophagus "},
        {"supercategory": "polyline", "id": 4, "name": "Z-line "},
        {"supercategory": "bounding_box", "id": 5, "name": "Stomach"},
        {"supercategory": "bounding_box", "id": 6, "name": "Antrum "},
        {"supercategory": "polygon", "id": 7, "name": "Incisura "},
        {"supercategory": "bounding_box", "id": 8, "name": "D1"},
        {"supercategory": "bounding_box", "id": 9, "name": "D2"},
        {"supercategory": "bounding_box", "id": 10, "name": "LES"},
        {"supercategory": "bounding_box", "id": 11, "name": "Pyloric opening "},
        {"supercategory": "bounding_box", "id": 12, "name": "Fundus "},
        {"supercategory": "point", "id": 13, "name": "point", "keypoints": "keypoint", "skeleton": []},
    ],
    "images": [
        {
            "coco_url": "cord-videos-dev/lFW59RQ9jcT4vHZeG14m8QWJKug1/cd63592b-a9e4-47d7-8563-aaed75c5afdc",
            "id": 0,
            "video_title": "failing_video_new.mp4",
            "file_name": "videos/cd63592b-a9e4-47d7-8563-aaed75c5afdc/0.jpg",
            "height": 480,
            "width": 640,
        },
        {
            "coco_url": "cord-videos-dev/lFW59RQ9jcT4vHZeG14m8QWJKug1/cd63592b-a9e4-47d7-8563-aaed75c5afdc",
            "id": 1,
            "video_title": "failing_video_new.mp4",
            "file_name": "videos/cd63592b-a9e4-47d7-8563-aaed75c5afdc/1.jpg",
            "height": 480,
            "width": 640,
        },
        {
            "coco_url": "cord-videos-dev/lFW59RQ9jcT4vHZeG14m8QWJKug1/cd63592b-a9e4-47d7-8563-aaed75c5afdc",
            "id": 2,
            "video_title": "failing_video_new.mp4",
            "file_name": "videos/cd63592b-a9e4-47d7-8563-aaed75c5afdc/2.jpg",
            "height": 480,
            "width": 640,
        },
        {
            "coco_url": "cord-videos-dev/lFW59RQ9jcT4vHZeG14m8QWJKug1/cd63592b-a9e4-47d7-8563-aaed75c5afdc",
            "id": 3,
            "video_title": "failing_video_new.mp4",
            "file_name": "videos/cd63592b-a9e4-47d7-8563-aaed75c5afdc/3.jpg",
            "height": 480,
            "width": 640,
        },
    ],
    "annotations": [
        {
            "area": 2601.9041279999997,
            "bbox": [189.696, 120.768, 54.976, 47.327999999999996],
            "category_id": 1,
            "image_id": 0,
            "iscrowd": 0,
            "segmentation": [[189.696, 120.768, 244.672, 120.768, 244.672, 168.096, 189.696, 168.096]],
            "keypoints": None,
            "num_keypoints": None,
            "id": 0,
            "attributes": {
                "track_id": 0,
                "encord_track_uuid": "428fba2b",
                "classifications": {},
                "manual_annotation": True,
            },
        },
        {
            "area": 0.0,
            "bbox": [271.552, 165.984, 0.0, 0.0],
            "category_id": 13,
            "image_id": 0,
            "iscrowd": 0,
            "segmentation": [[271.552, 165.984]],
            "keypoints": [271.552, 165.984, 2.0],
            "num_keypoints": 1,
            "id": 1,
            "attributes": {
                "track_id": 1,
                "encord_track_uuid": "82724bb2",
                "classifications": {},
                "manual_annotation": True,
            },
        },
        {
            "area": 999.9974399999999,
            "bbox": [195.32800000000003, 238.512, 40.064, 24.959999999999997],
            "category_id": 1,
            "image_id": 0,
            "iscrowd": 0,
            "segmentation": [
                [
                    195.32800000000003,
                    238.512,
                    235.39200000000002,
                    238.512,
                    235.39200000000002,
                    263.472,
                    195.32800000000003,
                    263.472,
                ]
            ],
            "keypoints": None,
            "num_keypoints": None,
            "id": 2,
            "attributes": {
                "track_id": 2,
                "encord_track_uuid": "u8BIOLmY",
                "classifications": {},
                "manual_annotation": True,
            },
        },
        {
            "area": 2768.498688,
            "bbox": [340.032, 283.872, 63.104, 43.872],
            "category_id": 1,
            "image_id": 0,
            "iscrowd": 0,
            "segmentation": [
                [340.032, 283.872, 403.13599999999997, 283.872, 403.13599999999997, 327.744, 340.032, 327.744]
            ],
            "keypoints": None,
            "num_keypoints": None,
            "id": 3,
            "attributes": {
                "track_id": 3,
                "encord_track_uuid": "/ghwDXVK",
                "classifications": {},
                "manual_annotation": True,
            },
        },
        {
            "area": 2538.319872,
            "bbox": [169.024, 116.016, 54.016000000000005, 46.992],
            "category_id": 1,
            "image_id": 1,
            "iscrowd": 0,
            "segmentation": [
                [169.024, 116.016, 223.04000000000002, 116.016, 223.04000000000002, 163.008, 169.024, 163.008]
            ],
            "keypoints": None,
            "num_keypoints": None,
            "id": 4,
            "attributes": {
                "track_id": 0,
                "encord_track_uuid": "428fba2b",
                "classifications": {},
                "manual_annotation": False,
            },
        },
        {
            "area": 999.9974399999999,
            "bbox": [195.32800000000003, 238.512, 40.064, 24.959999999999997],
            "category_id": 1,
            "image_id": 1,
            "iscrowd": 0,
            "segmentation": [
                [
                    195.32800000000003,
                    238.512,
                    235.39200000000002,
                    238.512,
                    235.39200000000002,
                    263.472,
                    195.32800000000003,
                    263.472,
                ]
            ],
            "keypoints": None,
            "num_keypoints": None,
            "id": 5,
            "attributes": {
                "track_id": 2,
                "encord_track_uuid": "u8BIOLmY",
                "classifications": {},
                "manual_annotation": True,
            },
        },
        {
            "area": 2538.319872,
            "bbox": [169.024, 114.0, 54.016000000000005, 46.992],
            "category_id": 1,
            "image_id": 2,
            "iscrowd": 0,
            "segmentation": [
                [169.024, 114.0, 223.04000000000002, 114.0, 223.04000000000002, 160.992, 169.024, 160.992]
            ],
            "keypoints": None,
            "num_keypoints": None,
            "id": 6,
            "attributes": {
                "track_id": 0,
                "encord_track_uuid": "428fba2b",
                "classifications": {},
                "manual_annotation": False,
            },
        },
        {
            "area": 2538.319872,
            "bbox": [171.968, 114.0, 54.016000000000005, 46.992],
            "category_id": 1,
            "image_id": 3,
            "iscrowd": 0,
            "segmentation": [
                [171.968, 114.0, 225.98399999999998, 114.0, 225.98399999999998, 160.992, 171.968, 160.992]
            ],
            "keypoints": None,
            "num_keypoints": None,
            "id": 7,
            "attributes": {
                "track_id": 0,
                "encord_track_uuid": "428fba2b",
                "classifications": {},
                "manual_annotation": False,
            },
        },
    ],
}

# This test is disabled because it fails when executing tests in parallel
# def test_coco_exporter_without_coco_extra():
#     # Simulate the absence of `pycocotools` and `shapely` packages
#     with patch.dict("sys.modules", {"pycocotools": None, "shapely": None}):
#         with pytest.raises(ImportError, match="The 'pycocotools' and 'shapely' packages are required"):
#             from encord.utilities.coco.exporter import CocoExporter
#
#             output = CocoExporter([BASE_LABEL_DICT], ontology_structure).export()
#             assert output == EXPECTED_COCO_RESULT


def test_coco_exporter_with_coco_extra():
    from encord.utilities.coco.exporter import CocoExporter

    output = CocoExporter([BASE_LABEL_DICT], ontology_structure).export()
    assert output == EXPECTED_COCO_RESULT



import ast
import os
import subprocess

# Clone the repositories only if they don't exist
if not os.path.exists("encord-client-python"):
    subprocess.run(["git", "clone", "https://github.com/encord-team/encord-client-python.git"])
if not os.path.exists("encord-docs-mint"):
    subprocess.run(["git", "clone", "https://github.com/encord-team/encord-docs-mint.git"])

# Now set the paths
source_repo_path = "encord-client-python"  # After cloning, the repo will be in this directory
destination_repo_path = "encord-docs-mint"  # Same for the destination repo

# Folder paths inside the repos
source_folder = "tests/docs"
destination_folder = "snippets/SDKCodeExamples"

# Define the paths to the Python files in the source repo and the corresponding .mdx output in the destination repo
python_files = [
    os.path.join(source_repo_path, source_folder, file)  # All Python files in the source folder
    for file in os.listdir(os.path.join(source_repo_path, source_folder))
    if file.endswith(".py")  # Only select .py files
]

# Ensure the destination folder exists in repo2
destination_folder_path = os.path.join(destination_repo_path, destination_folder)
if not os.path.exists(destination_folder_path):
    os.makedirs(destination_folder_path)


# Function to extract the code block name from a Python file's docstring
def get_code_block_name(python_file_path):
    with open(python_file_path, "r") as file:
        # Parse the Python file to extract the docstring using AST
        tree = ast.parse(file.read())
        if isinstance(tree.body[0], ast.Expr) and isinstance(tree.body[0].value, ast.Str):
            return tree.body[0].value.splitlines()[0].replace("Code Block Name:", "").strip()
    return "Untitled Code Block"


# Loop through each Python file and create a corresponding .mdx file
for python_file_path in python_files:
    # Ensure the source file exists before reading it
    if not os.path.exists(python_file_path):
        print(f"Error: Python file not found at {python_file_path}")
        continue

    # Read the content of the Python file
    with open(python_file_path, "r") as python_file:
        python_content = python_file.read()

    # Extract the code block name from the docstring
    code_block_name = get_code_block_name(python_file_path)

    # Create a corresponding .mdx file in the destination repo
    mdx_file_name = (
        os.path.splitext(os.path.basename(python_file_path))[0] + ".mdx"
    )  # Use the Python file name, but with .mdx extension
    mdx_file_path = os.path.join(destination_folder_path, mdx_file_name)

    # Initialize the .mdx file with the Python code inside a code block, and add the tab name from the docstring
    mdx_content = f"```python {code_block_name}\n{python_content}\n```"

    # Write the content to the corresponding .mdx file
    with open(mdx_file_path, "w") as mdx_file:
        mdx_file.write(mdx_content)

    print(
        f"Python code from {python_file_path} has been successfully written to {mdx_file_path} with tab name '{code_block_name}'"
    )



"""
Code Block Name: View label logs
"""

# Import dependencies
from encord import EncordUserClient

# User input
SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
DATA_UNIT_ID = "00000000-0000-0000-0000-000000000000"  # The unique identifier (data_hash) for the data unit


# Create user client using SSH key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Open the project you want to work on by specifying the Project ID
project = user_client.get_project(PROJECT_ID)

# Check if the method get_label_logs exists in project
if hasattr(project, "get_label_logs"):
    try:
        # Fetch the logs for the given DATA_UNIT_ID
        logs = project.get_label_logs(data_hash=DATA_UNIT_ID)

        # Check if logs were returned
        if logs:
            for log in logs:
                print(log)
                break  # print the first log only
        else:
            print("No logs found for this data unit.")
    except Exception as e:
        print(f"An error occurred while fetching logs: {e}")
else:
    print("The project does not have a method to fetch label logs.")



"""
Code Block Name: Keypoints Images/Videos
"""

# Import dependencies
from pathlib import Path

from encord import EncordUserClient, Project
from encord.objects import ChecklistAttribute, Object, ObjectInstance, Option, RadioAttribute, TextAttribute
from encord.objects.coordinates import PointCoordinate

SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"
# SSH_PATH = get_ssh_key() # replace it with ssh key
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
BUNDLE_SIZE = 100

# Create user client using ssh key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Get project for which labels are to be added
project: Project = user_client.get_project(PROJECT_ID)
assert project is not None, "Project not found — check PROJECT_ID"

# Get ontology structure
ontology_structure = project.ontology_structure
assert ontology_structure is not None, "Ontology structure is missing in the project"

# Get keypoint object for floral axis
keypoint_ontology_object: Object = ontology_structure.get_child_by_title(title="Floral axis", type_=Object)
assert keypoint_ontology_object is not None, "Keypoint object 'Floral axis' not found in ontology"

# Get radio attribute for floral axis type
floral_axis_type_radio_attribute = ontology_structure.get_child_by_title(type_=RadioAttribute, title="Type?")
assert floral_axis_type_radio_attribute is not None, "Radio attribute 'Type?' not found"

# Get radio options
pedicel_option = floral_axis_type_radio_attribute.get_child_by_title(type_=Option, title="Pedicel")
assert pedicel_option is not None, "Option 'Pedicel' not found under radio attribute 'Type?'"

peduncle_option = floral_axis_type_radio_attribute.get_child_by_title(type_=Option, title="Peduncle")
assert peduncle_option is not None, "Option 'Peduncle' not found under radio attribute 'Type?'"

other_floral_axis_option = floral_axis_type_radio_attribute.get_child_by_title(
    type_=Option, title="Other floral axis type"
)
assert other_floral_axis_option is not None, "Option 'Other floral axis type' not found under radio attribute 'Type?'"

# Get checklist attributes and options for Pedicel
pedicel_checklist_attribute = ontology_structure.get_child_by_title(
    type_=ChecklistAttribute, title="Pedicel Qualities?"
)
assert pedicel_checklist_attribute is not None, "Checklist attribute 'Pedicel Qualities?' not found"

pedicel_robust_option = pedicel_checklist_attribute.get_child_by_title(type_=Option, title="Robust")
assert pedicel_robust_option is not None, "Option 'Robust' not found under 'Pedicel Qualities?'"

pedicel_healthy_option = pedicel_checklist_attribute.get_child_by_title(type_=Option, title="Healthy")
assert pedicel_healthy_option is not None, "Option 'Healthy' not found under 'Pedicel Qualities?'"

pedicel_growth_alignment_option = pedicel_checklist_attribute.get_child_by_title(
    type_=Option, title="Good Growth and Alignment"
)
assert pedicel_growth_alignment_option is not None, (
    "Option 'Good Growth and Alignment' not found under 'Pedicel Qualities?'"
)

# Get checklist attributes and options for Peduncle
peduncle_checklist_attribute = ontology_structure.get_child_by_title(
    type_=ChecklistAttribute, title="Peduncle Qualities?"
)
assert peduncle_checklist_attribute is not None, "Checklist attribute 'Peduncle Qualities?' not found"

peduncle_robust_option = peduncle_checklist_attribute.get_child_by_title(type_=Option, title="Robust")
assert peduncle_robust_option is not None, "Option 'Robust' not found under 'Peduncle Qualities?'"

peduncle_healthy_option = peduncle_checklist_attribute.get_child_by_title(type_=Option, title="Healthy")
assert peduncle_healthy_option is not None, "Option 'Healthy' not found under 'Peduncle Qualities?'"

peduncle_growth_alignment_option = peduncle_checklist_attribute.get_child_by_title(
    type_=Option, title="Good Growth and Alignment"
)
assert peduncle_growth_alignment_option is not None, (
    "Option 'Good Growth and Alignment' not found under 'Peduncle Qualities?'"
)

# Get text attribute for specifying other floral axis types
other_floral_axis_option_text_attribute = ontology_structure.get_child_by_title(
    type_=TextAttribute, title="Specify floral axis type"
)
assert other_floral_axis_option_text_attribute is not None, "Text attribute 'Specify floral axis type' not found"

# Dictionary of labels per data unit and per frame with floral axis type specified, including quality options
video_image_frame_labels = {
    "blueberries-001.jpg": {
        0: {
            "label_ref": "floral_axis_001",
            "coordinates": PointCoordinate(x=0.01, y=0.02),
            "floral_axis_type": "Pedicel",
            "pedicel_quality_options": "Robust, Healthy",
        }
    },
    "persimmons-010.jpg": {
        0: [
            {
                "label_ref": "floral_axis_002",
                "coordinates": PointCoordinate(x=0.03, y=0.03),
                "floral_axis_type": "Peduncle",
                "peduncle_quality_options": "Robust, Healthy, Good Growth and Alignment",
            },
            {
                "label_ref": "floral_axis_003",
                "coordinates": PointCoordinate(x=0.05, y=0.04),
                "floral_axis_type": "Peduncle",
                "peduncle_quality_options": "Robust",
            },
            {
                "label_ref": "floral_axis_004",
                "coordinates": PointCoordinate(x=0.09, y=0.03),
                "floral_axis_type": "Other floral axis type",
                "Specify floral axis type": "Calyx",
            },
        ],
    },
    "blueberries-ig": {
        0: {
            "label_ref": "floral_axis_005",
            "coordinates": PointCoordinate(x=0.05, y=0.02),
            "floral_axis_type": "Pedicel",
            "pedicel_quality_options": "Robust, Healthy",
        },
        2: [
            {
                "label_ref": "floral_axis_006",
                "coordinates": PointCoordinate(x=0.03, y=0.03),
                "floral_axis_type": "Pedicel",
                "pedicel_quality_options": "Good Growth and Alignment",
            },
            {
                "label_ref": "floral_axis_007",
                "coordinates": PointCoordinate(x=0.04, y=0.05),
                "floral_axis_type": "Pedicel",
                "pedicel_quality_options": "Robust",
            },
            {
                "label_ref": "floral_axis_008",
                "coordinates": PointCoordinate(x=0.11, y=0.02),
                "floral_axis_type": "Other floral axis type",
                "Specify floral axis type": "Calyx",
            },
        ],
    },
    "persimmons-is": {
        0: {
            "label_ref": "floral_axis_009",
            "coordinates": PointCoordinate(x=0.01, y=0.02),
            "floral_axis_type": "Peduncle",
            "peduncle_quality_options": "Robust",
        },
        3: [
            {
                "label_ref": "floral_axis_010",
                "coordinates": PointCoordinate(x=0.03, y=0.03),
                "floral_axis_type": "Peduncle",
                "peduncle_quality_options": "Robust, Healthy, Good Growth and Alignment",
            },
            {
                "label_ref": "floral_axis_011",
                "coordinates": PointCoordinate(x=0.08, y=0.05),
                "floral_axis_type": "Peduncle",
                "peduncle_quality_options": "Robust",
            },
            {
                "label_ref": "floral_axis_012",
                "coordinates": PointCoordinate(x=0.11, y=0.02),
                "floral_axis_type": "Other floral axis type",
                "Specify floral axis type": "Calyx",
            },
        ],
    },
    "blueberries-vid-001.mp4": {
        103: [
            {
                "label_ref": "floral_axis_013",
                "coordinates": PointCoordinate(x=0.02, y=0.01),
                "floral_axis_type": "Pedicel",
                "pedicel_quality_options": "Robust",
            },
            {
                "label_ref": "floral_axis_014",
                "coordinates": PointCoordinate(x=0.06, y=0.06),
                "floral_axis_type": "Pedicel",
                "pedicel_quality_options": "Robust, Healthy, Good Growth and Alignment",
            },
            {
                "label_ref": "floral_axis_015",
                "coordinates": PointCoordinate(x=0.10, y=0.01),
                "floral_axis_type": "Other floral axis type",
                "Specify floral axis type": "Calyx",
            },
        ],
        104: [
            {
                "label_ref": "floral_axis_016",
                "coordinates": PointCoordinate(x=0.04, y=0.01),
                "floral_axis_type": "Pedicel",
                "pedicel_quality_options": "Robust",
            },
            {
                "label_ref": "floral_axis_014",
                "coordinates": PointCoordinate(x=0.08, y=0.05),
                "floral_axis_type": "Pedicel",
                "pedicel_quality_options": "Robust, Healthy, Good Growth and Alignment",
            },
            {
                "label_ref": "floral_axis_017",
                "coordinates": PointCoordinate(x=0.11, y=0.02),
                "floral_axis_type": "Other floral axis type",
                "Specify floral axis type": "Calyx",
            },
        ],
        105: [
            {
                "label_ref": "floral_axis_016",
                "coordinates": PointCoordinate(x=0.05, y=0.02),
                "floral_axis_type": "Pedicel",
                "pedicel_quality_options": "Robust",
            },
            {
                "label_ref": "floral_axis_014",
                "coordinates": PointCoordinate(x=0.07, y=0.04),
                "floral_axis_type": "Pedicel",
                "pedicel_quality_options": "Robust, Healthy, Good Growth and Alignment",
            },
            {
                "label_ref": "floral_axis_017",
                "coordinates": PointCoordinate(x=0.09, y=0.03),
                "floral_axis_type": "Other floral axis type",
                "Specify floral axis type": "Calyx",
            },
        ],
    },
}


# Cache initialized label rows
label_row_map = {}

# Step 1: Initialize all label rows using a bundle
with project.create_bundle(bundle_size=BUNDLE_SIZE) as bundle:
    for data_unit in video_image_frame_labels.keys():
        label_rows = project.list_label_rows_v2(data_title_eq=data_unit)
        assert isinstance(label_rows, list), f"Expected list of label rows for '{data_unit}', got {type(label_rows)}"

        if not label_rows:
            print(f"Skipping: No label row found for {data_unit}")
            continue

        label_row = label_rows[0]
        label_row.initialise_labels(bundle=bundle)
        assert label_row.ontology_structure is not None, f"Ontology not initialized for label row: {data_unit}"

        label_row_map[data_unit] = label_row  # Cache initialized label row for later use

# Step 2: Process all frames/annotations and prepare label rows to save
label_rows_to_save = []

for data_unit, frame_coordinates in video_image_frame_labels.items():
    label_row = label_row_map.get(data_unit)
    if not label_row:
        print(f"Skipping: No initialized label row found for {data_unit}")
        continue

    object_instances_by_label_ref = {}

    # Loop through the frames for the current data unit
    for frame_number, items in frame_coordinates.items():
        assert isinstance(frame_number, int), f"Frame number must be int, got {type(frame_number)}"
        if not isinstance(items, list):
            items = [items]

        for item in items:
            label_ref = item["label_ref"]
            coord = item["coordinates"]
            floral_axis_type = item["floral_axis_type"]

            assert floral_axis_type in {
                "Pedicel",
                "Peduncle",
                "Other floral axis type",
            }, f"Unexpected floral axis type '{floral_axis_type}' in data unit '{data_unit}'"

            # Check if label_ref already exists for reusability
            if label_ref not in object_instances_by_label_ref:
                keypoint_object_instance: ObjectInstance = keypoint_ontology_object.create_instance()
                assert keypoint_object_instance is not None, "Failed to create ObjectInstance"

                object_instances_by_label_ref[label_ref] = keypoint_object_instance
                checklist_attribute = None

                # Set floral axis type attribute
                if floral_axis_type == "Pedicel":
                    assert pedicel_option is not None, "Missing 'pedicel_option'"
                    keypoint_object_instance.set_answer(
                        attribute=floral_axis_type_radio_attribute, answer=pedicel_option
                    )
                    checklist_attribute = pedicel_checklist_attribute
                elif floral_axis_type == "Peduncle":
                    assert peduncle_option is not None, "Missing 'peduncle_option'"
                    keypoint_object_instance.set_answer(
                        attribute=floral_axis_type_radio_attribute, answer=peduncle_option
                    )
                    checklist_attribute = peduncle_checklist_attribute
                elif floral_axis_type == "Other floral axis type":
                    assert other_floral_axis_option is not None, "Missing 'other_floral_axis_option'"
                    keypoint_object_instance.set_answer(
                        attribute=floral_axis_type_radio_attribute, answer=other_floral_axis_option
                    )
                    text_answer = item.get("Specify floral axis type", "")
                    assert isinstance(text_answer, str), "'Specify floral axis type' must be a string"
                    keypoint_object_instance.set_answer(
                        attribute=other_floral_axis_option_text_attribute,
                        answer=text_answer,
                    )

                # Set checklist attributes
                checklist_answers = []
                quality_key = f"{floral_axis_type.lower()}_quality_options"
                quality_options = item.get(quality_key, "").split(", ")

                for quality in quality_options:
                    if quality == "Robust":
                        checklist_answers.append(
                            pedicel_robust_option if floral_axis_type == "Pedicel" else peduncle_robust_option
                        )
                    elif quality == "Healthy":
                        checklist_answers.append(
                            pedicel_healthy_option if floral_axis_type == "Pedicel" else peduncle_healthy_option
                        )
                    elif quality == "Good Growth and Alignment":
                        checklist_answers.append(
                            pedicel_growth_alignment_option
                            if floral_axis_type == "Pedicel"
                            else peduncle_growth_alignment_option
                        )

                if checklist_attribute and checklist_answers:
                    keypoint_object_instance.set_answer(
                        attribute=checklist_attribute, answer=checklist_answers, overwrite=True
                    )
            else:
                # Reuse existing instance across frames
                keypoint_object_instance = object_instances_by_label_ref[label_ref]

            # Assign the object to the frame and track it
            keypoint_object_instance.set_for_frames(coordinates=coord, frames=frame_number)

    # Add object instances to label_row **only if they have frames assigned**
    for keypoint_object_instance in object_instances_by_label_ref.values():
        assert isinstance(keypoint_object_instance, ObjectInstance), "Expected ObjectInstance type"
        if keypoint_object_instance.get_annotation_frames():
            label_row.add_object_instance(keypoint_object_instance)

    label_rows_to_save.append(label_row)

# Step 3: Save all label rows using a bundle
with project.create_bundle(bundle_size=BUNDLE_SIZE) as bundle:
    for label_row in label_rows_to_save:
        assert label_row is not None, "Trying to save a None label row"
        label_row.save(bundle=bundle)
        print(f"Saved label row for {label_row.data_title}")

print("Labels with floral axis type radio buttons, checklist attributes, and text labels added for all data units.")



from datetime import datetime
from unittest.mock import Mock, PropertyMock

import pytest
from deepdiff import DeepDiff

from encord.objects import LabelRowV2, Object, ObjectInstance, Shape
from encord.objects.frames import Range
from encord.objects.metadata import DICOMSeriesMetadata, DICOMSliceMetadata
from encord.ontology import Ontology
from encord.orm.label_row import AnnotationTaskStatus, LabelRowMetadata, LabelStatus
from tests.objects.data.audio_labels import EMPTY_AUDIO_LABELS
from tests.objects.data.dicom_labels_with_metadata import (
    DICOM_LABELS_WITH_METADATA_TEST_BLURB,
)


@pytest.fixture
def ontology() -> Ontology:
    get_child_by_hash = PropertyMock(
        return_value=Object(
            uid=1, name="Box", color="#D33115", shape=Shape.BOUNDING_BOX, feature_node_hash="MjI2NzEy", attributes=[]
        )
    )
    ontology_structure = Mock(get_child_by_hash=get_child_by_hash)
    yield Mock(structure=ontology_structure)


@pytest.fixture
def label_row_metadata() -> LabelRowMetadata:
    return LabelRowMetadata(
        label_hash="",
        branch_name="main",
        created_at=datetime.now(),
        last_edited_at=datetime.now(),
        data_hash="",
        data_title="",
        data_type="DICOM",
        data_link="",
        dataset_hash="",
        dataset_title="",
        label_status=LabelStatus.NOT_LABELLED,
        annotation_task_status=AnnotationTaskStatus.QUEUED,
        workflow_graph_node=None,
        is_shadow_data=False,
        duration=100,
        frames_per_second=25,
        number_of_frames=55,
        height=512,
        width=512,
        audio_codec=None,
        audio_sample_rate=None,
        audio_num_channels=None,
        audio_bit_depth=None,
    )


def test_label_row_metadata_accessor(ontology, label_row_metadata):
    label_row = LabelRowV2(label_row_metadata, Mock(), ontology)
    label_row.from_labels_dict(DICOM_LABELS_WITH_METADATA_TEST_BLURB)

    row_metadata = label_row.metadata
    assert row_metadata is not None
    assert isinstance(row_metadata, DICOMSeriesMetadata)

    for frame_view in label_row.get_frame_views():
        frame_metadata = frame_view.metadata
        assert frame_metadata is not None
        assert isinstance(frame_metadata, DICOMSliceMetadata)


def test_label_row_audio_metadata_accessor(ontology, label_row_metadata):
    label_row = LabelRowV2(label_row_metadata, Mock(), ontology)
    label_row.from_labels_dict(EMPTY_AUDIO_LABELS)

    assert label_row.audio_codec == "mp3"
    assert label_row.audio_num_channels == 2
    assert label_row.audio_sample_rate == 44100
    assert label_row.audio_bit_depth == 8


def test_checklist_parsing_merging_single_frame_events(ontology, label_row_metadata):
    answer_ranges = [
        (Range(start=536, end=536), ["abcDeFGH"]),
        (Range(start=1695, end=1695), ["abcDeFGH"]),
        (Range(start=541, end=541), ["xGvnZsqe"]),
        (Range(start=1730, end=1730), ["xGvnZsqe"]),
        (Range(start=1695, end=1695), ["lMT3lsvM"]),
    ]

    merged_result = ObjectInstance._merge_answers_to_non_overlapping_ranges(answer_ranges)
    expected_results = [
        (Range(start=536, end=536), {"abcDeFGH"}),
        (Range(start=541, end=541), {"xGvnZsqe"}),
        (Range(start=1695, end=1695), {"abcDeFGH", "lMT3lsvM"}),
        (Range(start=1730, end=1730), {"xGvnZsqe"}),
    ]

    assert not DeepDiff(merged_result, expected_results)


def test_checklist_parsing_merging_included_events(ontology, label_row_metadata):
    answer_ranges = [
        (Range(start=0, end=100), ["abcDeFGH"]),
        (Range(start=10, end=90), ["xGvnZsqe"]),
        (Range(start=20, end=80), ["nMakrEgd"]),
    ]

    merged_result = ObjectInstance._merge_answers_to_non_overlapping_ranges(answer_ranges)
    expected_results = [
        (Range(start=0, end=9), {"abcDeFGH"}),
        (Range(start=10, end=19), {"abcDeFGH", "xGvnZsqe"}),
        (Range(start=20, end=80), {"abcDeFGH", "xGvnZsqe", "nMakrEgd"}),
        (Range(start=81, end=90), {"abcDeFGH", "xGvnZsqe"}),
        (Range(start=91, end=100), {"abcDeFGH"}),
    ]

    assert not DeepDiff(merged_result, expected_results)


def test_checklist_parsing_merging_overlapped_events(ontology, label_row_metadata):
    answer_ranges = [
        (Range(start=0, end=60), ["abcDeFGH"]),
        (Range(start=40, end=80), ["xGvnZsqe"]),
        (Range(start=20, end=100), ["nMakrEgd"]),
    ]

    merged_result = ObjectInstance._merge_answers_to_non_overlapping_ranges(answer_ranges)
    expected_results = [
        (Range(start=0, end=19), {"abcDeFGH"}),
        (Range(start=20, end=39), {"abcDeFGH", "nMakrEgd"}),
        (Range(start=40, end=60), {"abcDeFGH", "nMakrEgd", "xGvnZsqe"}),
        (Range(start=61, end=80), {"xGvnZsqe", "nMakrEgd"}),
        (Range(start=81, end=100), {"nMakrEgd"}),
    ]

    assert not DeepDiff(merged_result, expected_results)



"""
Code Block Name: Classifications
"""

# Import dependencies
from pathlib import Path

from encord import EncordUserClient
from encord.objects import Classification, Option

SSH_PATH = "/Users/chris-encord/staging-sdk-ssh-key-private-key.txt"
# SSH_PATH = get_ssh_key() # replace it with ssh key
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
BUNDLE_SIZE = 100

# Create user client
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Get Project
project = user_client.get_project(PROJECT_ID)

# Define specific configurations for each data unit
data_unit_configs = [
    {
        "title": "anne-of-green-gables.pdf",
        "classifications": [
            {"name": "Accuracy", "type": "radio", "option": "5", "frame_range": (10, 20)},
            {"name": "Doc qualities", "type": "checklist", "options": ["Clarity", "Engaging"], "frame_range": (15, 25)},
        ],
    },
    {
        "title": "the-legend-of-sleepy-hollow.pdf",
        "classifications": [
            {"name": "Accuracy", "type": "radio", "option": "3", "frame_range": (5, 15)},
            {"name": "Edit", "type": "text", "text": "Needs more context", "frame_range": (3, 10)},
        ],
    },
    {
        "title": "the-iliad.pdf",
        "classifications": [
            {
                "name": "Doc qualities",
                "type": "checklist",
                "options": ["Logical structure", "Audience focused"],
                "frame_range": (30, 40),
            }
        ],
    },
]

# Lookup map for quicker saving later
label_row_map = {}

# Step 1: Fetch and initialize label rows in a bundle
with project.create_bundle(bundle_size=BUNDLE_SIZE) as init_bundle:
    for config in data_unit_configs:
        label_rows = project.list_label_rows_v2(data_title_eq=config["title"])

        assert isinstance(label_rows, list), (
            f"Expected a list of label rows for '{config['title']}', got {type(label_rows)}"
        )

        if not label_rows:
            print(f"No label row found for {config['title']}, skipping...")
            continue

        label_row = label_rows[0]
        label_row.initialise_labels(bundle=init_bundle)

        assert label_row.ontology_structure is not None, f"Ontology structure not initialized for '{config['title']}'"
        label_row_map[config["title"]] = label_row  # Save for later

# Step 2: Apply classifications
for config in data_unit_configs:
    try:
        label_row = label_row_map.get(config["title"])
        if not label_row:
            continue

        ontology_structure = label_row.ontology_structure
        assert ontology_structure is not None, f"Missing ontology structure for '{config['title']}'"

        for classification_config in config["classifications"]:
            classification = ontology_structure.get_child_by_title(
                title=classification_config["name"], type_=Classification
            )
            assert classification is not None, (
                f"Classification '{classification_config['name']}' not found in ontology for '{config['title']}'"
            )

            classification_instance = classification.create_instance()

            if classification_config["type"] == "radio":
                option = classification.get_child_by_title(title=classification_config["option"], type_=Option)
                assert option is not None, (
                    f"Radio option '{classification_config['option']}' not found in classification '{classification_config['name']}'"
                )
                classification_instance.set_answer(option)

            elif classification_config["type"] == "checklist":
                options = []
                for opt_title in classification_config["options"]:
                    option = classification.get_child_by_title(title=opt_title, type_=Option)
                    assert option is not None, (
                        f"Checklist option '{opt_title}' not found in classification '{classification_config['name']}'"
                    )
                    options.append(option)
                classification_instance.set_answer(options)

            elif classification_config["type"] == "text":
                assert "text" in classification_config, (
                    f"Missing 'text' field in text classification for '{config['title']}'"
                )
                classification_instance.set_answer(answer=classification_config["text"])

            else:
                raise ValueError(f"Unsupported classification type: {classification_config['type']}")

            start_frame, end_frame = classification_config["frame_range"]
            assert isinstance(start_frame, int) and isinstance(end_frame, int) and start_frame <= end_frame, (
                f"Invalid frame range {classification_config['frame_range']} for '{config['title']}'"
            )

            for frame in range(start_frame, end_frame + 1):
                classification_instance.set_for_frames(frames=frame, manual_annotation=True, confidence=1.0)

            label_row.add_classification_instance(classification_instance)

    except Exception as e:
        print(f"Error processing {config['title']}: {e}")

# Step 3: Save all label rows in a bundle
with project.create_bundle(bundle_size=BUNDLE_SIZE) as save_bundle:
    for label_row in label_row_map.values():
        assert label_row is not None, "Attempting to save an uninitialized label row"
        label_row.save(bundle=save_bundle)

print("All label rows processed and saved.")



import uuid
from unittest.mock import MagicMock

from pytest import fixture

from encord.orm.workflow import WorkflowDTO, WorkflowNode, WorkflowStageType
from encord.workflow import Workflow

WORKFLOW_ANNOTATION_UUID = uuid.uuid4()
WORKFLOW_REVIEW_UUID = uuid.uuid4()
WORKFLOW_COMPLETE_UUID = uuid.uuid4()

workflow_dto = WorkflowDTO(
    stages=[
        WorkflowNode(uuid=WORKFLOW_ANNOTATION_UUID, stage_type=WorkflowStageType.ANNOTATION, title="Annotation 1"),
        WorkflowNode(uuid=WORKFLOW_REVIEW_UUID, stage_type=WorkflowStageType.REVIEW, title="Review 1"),
        WorkflowNode(uuid=WORKFLOW_COMPLETE_UUID, stage_type=WorkflowStageType.DONE, title="Complete"),
    ]
)


@fixture
def workflow() -> Workflow:
    return Workflow(MagicMock(), uuid.uuid4(), workflow_dto)



from pathlib import Path

import prettyprinter as pp

from encord import EncordUserClient
from encord.utilities.client_utilities import CvatImporterSuccess, LocalImport

# User input
SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"

# Create user client using SSH key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Increase networking timeouts for this long running operation.
timeout = 1800
user_client.user_config.read_timeout = timeout
user_client.user_config.write_timeout = timeout
user_client.user_config.connect_timeout = timeout

# We have placed the unzipped Pizza Project directory into a
# `data` folder relative to this script
data_folder = "data/Pizza Project"
dataset_name = "Pizza Images Dataset"
cvat_importer_ret = user_client.create_project_from_cvat(LocalImport(file_path=data_folder), dataset_name)

# Check if the import was a success and inspect the return value
if type(cvat_importer_ret) == CvatImporterSuccess:
    print(f"project_hash = {cvat_importer_ret.project_hash}")
    print(f"dataset_hash = {cvat_importer_ret.dataset_hash}")
    pp.pprint(cvat_importer_ret.issues)



"""
Code Block Name: List all Projects
"""

# Import dependencies
from encord.user_client import EncordUserClient

# User input
SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"

# Create user client using SSH key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# List existing projects
projects = user_client.get_projects()
print(projects)



import json
from datetime import datetime
from unittest.mock import MagicMock, patch
from uuid import uuid4

import pytest
from requests import Session

from encord.configs import SshConfig
from encord.http.constants import RequestsSettings
from encord.http.v2.api_client import ApiClient
from encord.user_client import EncordUserClient
from tests.fixtures import PRIVATE_KEY, PRIVATE_KEY_PEM

PROJECT_HASH = uuid4().hex
ONTOLOGY_HASH = uuid4().hex
DATASET_HASH = uuid4().hex


@pytest.fixture
def api_client():
    return ApiClient(config=SshConfig(PRIVATE_KEY))


def stub_responses(*args, **kwargs) -> MagicMock:
    mock_response = MagicMock()

    if args[0].path_url.startswith("/v2/public/projects"):
        response = {
            "project_hash": PROJECT_HASH,
            "project_type": "manual_qa",
            "status": "inProgress",
            "title": "Test project",
            "description": "",
            "created_at": "2024-01-01 11:11:11",
            "last_edited_at": "2024-01-01 11:11:11",
            "editor_ontology": {},
            "datasets": [uuid4().hex],
            "label_rows": [],
            "ontology_hash": ONTOLOGY_HASH,
        }
        mock_response.status_code = 200
        mock_response.json.return_value = response
    elif args[0].path_url.startswith("/v2/public/ontologies"):
        response = {
            "ontologyUuid": ONTOLOGY_HASH,
            "title": "Test ontology",
            "description": "",
            "created_at": "2024-01-01 11:11:11",
            "last_edited_at": "2024-01-01 11:11:11",
            "editor": {"objects": [], "classifications": []},
            "user_role": 1,
        }
        mock_response.status_code = 200
        mock_response.json.return_value = response
    else:
        # Legacy api
        body_json = json.loads(args[0].body)
        request_type = body_json["query_type"]
        if request_type == "ontology":
            response = {
                "title": "Test ontology",
                "description": "",
                "ontology_hash": ONTOLOGY_HASH,
                "created_at": "2024-01-01 11:11:11",
                "last_edited_at": "2024-01-01 11:11:11",
                "editor": {
                    "objects": [],
                    "classifications": [],
                },
            }
        elif request_type == "dataset":
            response = {
                "dataset_hash": DATASET_HASH,
                "title": "Test dataset",
                "description": "",
                "dataset_type": "cord",
                "data_rows": [],
            }
        elif request_type in ["datarows", "labelrowmetadata"]:
            response = {}  # These objects are not used in tests, so it's ok to return empty descriptor for now
        else:
            assert False, f"Unsupported request: {request_type}"

        mock_response.status_code = 200
        mock_response.json.return_value = {"status": 200, "response": response}
        mock_response.content = json.dumps({"status": 200, "response": response})

    return mock_response


def verify_timeout_and_reset(mock: MagicMock, requests_settings: RequestsSettings, expected_call_count=1) -> None:
    assert mock.call_count == expected_call_count
    assert all(
        call[1]["timeout"] == (requests_settings.connect_timeout, requests_settings.read_timeout)
        for call in mock.call_args_list
    )

    mock.reset_mock()


@patch.object(Session, "send")
def test_request_timeout_settings_correctly_propagated(send: MagicMock, api_client: ApiClient):
    requests_settings = RequestsSettings(
        connect_timeout=1001,
        read_timeout=1002,
        write_timeout=1003,
    )

    send.side_effect = stub_responses

    user_client = EncordUserClient.create_with_ssh_private_key(
        ssh_private_key=PRIVATE_KEY_PEM, requests_settings=requests_settings
    )
    project = user_client.get_project(PROJECT_HASH)
    verify_timeout_and_reset(send, requests_settings, 2)

    project.list_label_rows_v2()
    verify_timeout_and_reset(send, requests_settings)

    dataset = user_client.get_dataset(DATASET_HASH)
    verify_timeout_and_reset(send, requests_settings)

    dataset.list_data_rows()
    verify_timeout_and_reset(send, requests_settings)

    ontology = user_client.get_ontology(ONTOLOGY_HASH)
    verify_timeout_and_reset(send, requests_settings)

    ontology.refetch_data()
    verify_timeout_and_reset(send, requests_settings)

    # Re-mocking for V2 API
    mock_response = MagicMock()
    mock_response.status_code = 200
    mock_response.json.return_value = {"results": []}
    mock_response.content = json.dumps({"results": []})
    send.side_effect = None
    send.return_value = mock_response

    _ = list(project.list_collaborator_timers(datetime.now()))
    verify_timeout_and_reset(send, requests_settings)



"""
Code Block Name: Move tasks to the next stage
"""

# Import dependencies
from encord import EncordUserClient, Project
from encord.workflow import AnnotationStage

# User input
SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
WORKFLOW_STAGE_NAME = "Annotate 1"
WORKFLOW_STAGE_TYPE = AnnotationStage
BUNDLE_SIZE = 100  # You can adjust this value as needed, but keep it <= 1000

# Create user client using SSH key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Open the project you want to work on by specifying the Project ID
project = user_client.get_project(PROJECT_ID)

# Get the specific stage (in this case, "Annotate 1")
stage = project.workflow.get_stage(name=WORKFLOW_STAGE_NAME, type_=WORKFLOW_STAGE_TYPE)

# Create a bundle and move tasks
with project.create_bundle(bundle_size=BUNDLE_SIZE) as bundle:
    for task in stage.get_tasks():
        # The task is submitted as the user who is currently assigned to the task.
        # With retain_assignee=True an error occurs if there are tasks without an assignee.
        task.submit(retain_assignee=True, bundle=bundle)
        print(f"Task: {task}")

print("All tasks have been processed and moved to the next stage.")



"""
Code Block Name: Add users to Project
"""

# Import dependencies
from encord import EncordUserClient, Project
from encord.utilities.project_user import ProjectUserRole

# User input
SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
USER_01 = "example-user-01@encord.com"  # Email address for user you want to add to Project
USER_02 = "example-user-02@encord.com"  # Email address for user you want to add to Project
USER_03 = "exmaple-user-02@encord.com"  # Email address for user you want to add to Project

# Create user client using SSH key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Open the project you want to work on by specifying the Project ID
project = user_client.get_project(PROJECT_ID)

# Add users by specifying their email addresses, as well as the role these users should have.
added_users = project.add_users(
    [USER_01, USER_02, USER_03],
    # Specify the role the user's have in the Project
    ProjectUserRole.ANNOTATOR,
)

# Print the new users added to the project
print(added_users)



import uuid
from datetime import datetime
from unittest.mock import MagicMock

from encord.http.bundle import Bundle
from encord.workflow import AnnotationStage, AnnotationTask, AnnotationTaskStatus, Workflow


def test_bulk_assign_annotations(workflow: Workflow) -> None:
    annotation_stage = workflow.get_stage(name="Annotation 1", type_=AnnotationStage)

    annotation_stage._workflow_client.api_client.get_paged_iterator.return_value = iter(
        [
            AnnotationTask(
                uuid=uuid.uuid4(),
                created_at=datetime.now(),
                updated_at=datetime.now(),
                data_hash=uuid.uuid4(),
                data_title=f"data unit {x}",
                label_branch_name="main",
                assignee=None,
                status=AnnotationTaskStatus.NEW,
            )
            for x in range(0, 3)
        ]
    )

    action_resource = MagicMock()
    annotation_stage._workflow_client.api_client.post = action_resource

    bundle = Bundle()
    for task in annotation_stage.get_tasks():
        task.assign("test-user@encord.com", bundle=bundle)

    assert not action_resource.called

    bundle.execute()

    assert action_resource.called
    assert isinstance(action_resource.call_args[1]["payload"], list)
    assert len(action_resource.call_args[1]["payload"]) == 3
